﻿' (C) Copyright 2002-2005 by Autodesk, Inc.
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted,
' provided that the above copyright notice appears in all copies and
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports System.Windows

Imports Autodesk.AutoCAD.Windows

Public Class adskClass

    ' Определяем команду 'helloworld'
    <CommandMethod("helloworld")> _
    Public Sub helloworld()

        ' Получаем объект редактора
        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
        ' Пишем в командной строке
        ed.WriteMessage(vbCr + "Здравствуй, Мир!")

    End Sub

    <CommandMethod("addAnEnt")> _
    Public Sub AddAnEnt()

        ' Получаем объект редактора т.к. собираемся кое-что вводить
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' уточним какой тип примитива будем создавать
        Dim getWhichEntityOptions As PromptKeywordOptions = New PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block")
        ' теперь запросим его
        Dim getWhichEntityResult As PromptResult = ed.GetKeywords(getWhichEntityOptions)
        ' Если OK
        If (getWhichEntityResult.Status = PromptStatus.OK) Then

            ' Проверим какой тип примитива требуется создать
            Select Case getWhichEntityResult.StringResult

                Case "Circle"

                    ' Укажем центр окружности
                    Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick Center Point : ")
                    Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)
                    ' Если OK
                    If (getPointResult.Status = PromptStatus.OK) Then

                        ' получаем радиус
                        Dim getRadiusOptions As PromptDistanceOptions = New PromptDistanceOptions("Pick Radius : ")
                        ' в качестве базовой точки - точку которую мы указали для центра
                        getRadiusOptions.BasePoint = getPointResult.Value
                        ' сообщим механизму ввода использовать базовую точку
                        getRadiusOptions.UseBasePoint = True
                        ' теперь получим радиус
                        Dim getRadiusResult As PromptDoubleResult = ed.GetDistance(getRadiusOptions)
                        ' Если всё OK
                        If (getRadiusResult.Status = PromptStatus.OK) Then

                            ' Нужно добавить окружность к текущему пространству
                            ' Получаем активную базу
                            Dim dwg As Database = ed.Document.Database
                            ' стартуем транзакцию
                            Dim trans As Transaction = dwg.TransactionManager.StartTransaction()
                            Try

                                ' создаём новую окружность
                                Dim circle As New Circle(getPointResult.Value, Vector3d.ZAxis, getRadiusResult.Value)
                                ' открываем активное пространство (запись таблицы блоков) для записи
                                Dim btr As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                                ' добавляем окружность к текущему пространству (скорее всего пространство модели)
                                btr.AppendEntity(circle)
                                ' сообщим транзакции о необходимости автозакрыть объект окружности
                                trans.AddNewlyCreatedDBObject(circle, True)
                                ' завершаеи (commit) транзакцию
                                trans.Commit()

                            Catch ex As Exception
                                ' Есть исключение - сообщим о нём
                                ed.WriteMessage("problem due to " + ex.Message)
                            Finally
                                ' Дело сделано, в любом случае освободим транзакцию
                                trans.Dispose()
                            End Try

                        End If
                    End If

                Case "Block"

                    ' вводим имя блока
                    Dim blockNameOptions As PromptStringOptions = New PromptStringOptions("Enter name of the Block to create : ")
                    ' запрещаем пробелы в имени блока
                    blockNameOptions.AllowSpaces = False
                    ' получаем имя
                    Dim blockNameResult As PromptResult = ed.GetString(blockNameOptions)
                    ' Если ok
                    If (blockNameResult.Status = PromptStatus.OK) Then

                        ' Создадим определение блока
                        ' получаем активный чертеж
                        Dim dwg As Database = ed.Document.Database
                        ' начнем транзакцию
                        Dim trans As Transaction = dwg.TransactionManager.StartTransaction
                        Try

                            ' создаем новое определение блока
                            Dim newBlockDef As BlockTableRecord = New BlockTableRecord
                            ' задаём имя блоку
                            newBlockDef.Name = blockNameResult.StringResult

                            ' добавим определение блока в таблицу блоков
                            ' откроем таблицу блоков для чтения и проверим есть ли уже такое имя блока в ней
                            Dim blockTable As BlockTable = trans.GetObject(dwg.BlockTableId, OpenMode.ForRead)
                            ' проверяем есть ли уже такое имя
                            If (blockTable.Has(blockNameResult.StringResult) = False) Then
                                ' Если нет, то можем добавить его
                                ' только сначала нужно переключится в режим для записи
                                blockTable.UpgradeOpen()
                                blockTable.Add(newBlockDef)
                                ' сообщаем менеджеру транзакций о новом объекте для автозакрытия
                                trans.AddNewlyCreatedDBObject(newBlockDef, True)
                            End If

                            ' теперь добавим несколько объектов к определению блока
                            Dim circle1 As New Circle(New Point3d(0, 0, 0), Vector3d.ZAxis, 10)
                            Dim circle2 As New Circle(New Point3d(20, 10, 0), Vector3d.ZAxis, 10)
                            Dim circle3 As New Circle(New Point3d(40, 20, 0), Vector3d.ZAxis, 10)
                            Dim circle4 As New Circle(New Point3d(60, 30, 0), Vector3d.ZAxis, 10)
                            Dim circle5 As New Circle(New Point3d(80, 40, 0), Vector3d.ZAxis, 10)
                            newBlockDef.AppendEntity(circle1)
                            newBlockDef.AppendEntity(circle2)
                            newBlockDef.AppendEntity(circle3)
                            newBlockDef.AppendEntity(circle4)
                            newBlockDef.AppendEntity(circle5)
                            ' сообщим менеджеру транзакций о новых объектах
                            trans.AddNewlyCreatedDBObject(circle1, True)
                            trans.AddNewlyCreatedDBObject(circle2, True)
                            trans.AddNewlyCreatedDBObject(circle3, True)
                            trans.AddNewlyCreatedDBObject(circle4, True)
                            trans.AddNewlyCreatedDBObject(circle5, True)

                            ' теперь вставим блок в текущее пространство - запросим точку вставки
                            Dim blockRefPointOptions As PromptPointOptions = New PromptPointOptions("Pick insertion point of Block : ")
                            Dim blockRefPointResult As PromptPointResult = ed.GetPoint(blockRefPointOptions)
                            ' если не всё в порядке
                            If (blockRefPointResult.Status <> PromptStatus.OK) Then
                                ' освобождаем транзакцию, чтобы отменить изменения и выходим
                                trans.Dispose()
                                Return
                            End If
                            ' теперь у нас есть определение блока и точка для вставки - создадим вставку блока
                            Dim blockRef As BlockReference = New BlockReference(blockRefPointResult.Value, newBlockDef.ObjectId)
                            ' открываем текущее пространство для записи и добавим вставку блока в него
                            Dim curSpace As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                            ' добавляем вставку блока
                            curSpace.AppendEntity(blockRef)
                            ' укажем транзакции о новом объекте для автозакрытия
                            trans.AddNewlyCreatedDBObject(blockRef, True)

                            ' Если всё в норме - завершим транзакцию
                            trans.Commit()

                        Catch ex As Exception
                            ' Есть проблема - сообщим о ней
                            ed.WriteMessage("a problem occured because " + ex.Message)
                        Finally
                            ' чтобы не произошло, а транзакцию нужно освободить
                            trans.Dispose()

                        End Try

                    End If

            End Select
        End If

    End Sub

    ' Объявим объект PaletteSset - он должен быть создан только один раз
    Public myPaletteSet As Autodesk.AutoCAD.Windows.PaletteSet
    ' Нам нужна палитра, которая размещается на PaletteSet
    Public myPalette As UserControl1

    ' команда palette
    <CommandMethod("palette")> _
    Public Sub palette()

        ' проверим создана ли уже
        If (myPaletteSet = Nothing) Then
            ' создаём PaletteSet с уникальным GUID
            myPaletteSet = New Autodesk.AutoCAD.Windows.PaletteSet("My Palette", New Guid("D61D0875-A507-4b73-8B5F-9266BEACD596"))
            ' создаём новую палитру, внутри которой контрол дерево (Tree Control)
            myPalette = New UserControl1
            ' теперь добавим palette к paletteset
            myPaletteSet.Add("Palette1", myPalette)

        End If

        ' сделаем видимым paletteset
        myPaletteSet.Visible = True

    End Sub

    <CommandMethod("addDBEvents")> _
    Public Sub addDBEvents()

        ' нужно предварительно создать палитру
        If myPalette Is Nothing Then
            ' получаем объект редактора
            Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
            ' пишем в командной строке
            ed.WriteMessage(vbCr + "Please call the 'palette' command first")
            Exit Sub
        End If

        ' получаем активную базу
        Dim curDwg As Database = Application.DocumentManager.MdiActiveDocument.Database
        ' добавим обработчики событий
        AddHandler curDwg.ObjectAppended, New ObjectEventHandler(AddressOf callback_ObjectAppended)
        AddHandler curDwg.ObjectErased, New ObjectErasedEventHandler(AddressOf callback_ObjectErased)
        AddHandler curDwg.ObjectReappended, New ObjectEventHandler(AddressOf callback_ObjectReappended)
        AddHandler curDwg.ObjectUnappended, New ObjectEventHandler(AddressOf callback_ObjectUnappended)

    End Sub

    Private Sub callback_ObjectAppended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' добавим имя класса объекта к tree view
        Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
        ' записываем ID объекта для того, чтобы его можно было потом найти
        newNode.Tag = e.DBObject.ObjectId.ToString()

    End Sub

    Private Sub callback_ObjectErased(ByVal sender As Object, ByVal e As ObjectErasedEventArgs)

        ' Если объект удалён
        If e.Erased Then
            ' находим объект в контроле treeview чтобы удалить его
            For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
                ' если это он
                If (node.Tag = e.DBObject.ObjectId.ToString) Then
                    node.Remove()
                    Exit For
                End If
            Next
            ' если объект восстановлен
        Else
            ' добавим имя класса объекта к tree view
            Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
            ' записываем ID объекта для того, чтобы его можно было потом найти
            newNode.Tag = e.DBObject.ObjectId.ToString()
        End If
    End Sub
    Private Sub callback_ObjectReappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' добавим имя класса объекта к tree view
        Dim newNode As Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
        ' записываем ID объекта для того, чтобы его можно было потом найти
        newNode.Tag = e.DBObject.ObjectId.ToString()

    End Sub

    Private Sub callback_ObjectUnappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' находим объект в контроле treeview чтобы удалить его
        For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
            ' если это он
            If (node.Tag = e.DBObject.ObjectId.ToString) Then
                node.Remove()
                Exit For
            End If
        Next

    End Sub


    <CommandMethod("addData")> _
    Public Sub addData()

        ' получаем объект редактора
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ' указываем примитив, для добавления в него данных!
        Dim getEntityResult As PromptEntityResult = ed.GetEntity("Pick an entity to add an Extension Dictionary to : ")
        ' Если всё OK
        If (getEntityResult.Status = PromptStatus.OK) Then
            ' стартуем транзакцию
            Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
            Try

                ' Начало Lab5
                ' Здесь мы добавим данные к выбранному примитиву.

                ' 1. Объявим переменную типа Entity с именем ent. Создадим её
                ' методом GetOBject объекта Transaction созданного выше. (Trans)
                ' В качестве параметра ObjectId используем свойство ObjectId
                ' PromptEntityResult созданного выше.(getEntityResult)
                ' Объект открываем для чтения.

                ' 2. Используем оператор "If Then" и проверяем свойство IsNull
                ' для свойства ExtensionDictionary примитива ent. Если Null
                ' тогда создаём ExtensionDictionary.
                ' Заметка: Поместим "End If" после шага 4.


                ' 3. Выполним метод UpgradeOpen примитива. Т.к. у него нет
                ' расширенного словаря, мы хотим его добавить.
                ' Для этого примитив должен быть открыт для записи

                ' 4. Создадим ExtensionDictionary при помощи вызова метода
                ' CreateExtensionDictionary примитива.

                ' 5. Объявим переменную типа DBDictionary. Создадим её
                ' методом GetObject объекта Transaction созданного выше.
                ' (trans). В качестве параметра ObjectId используем свойство
                ' ExtensionDictionary примитива ent, созданного на шаге 1.
                ' Откроем его для чтения

                ' 6. Проверим, нет ли еще в словаре такого объекта, который мы хотим добавить
                ' Воспользуемся свойством Contains словаря в операторе "If Else
                ' End If".
                ' Заметка: Поместим "Else" после шага 12, а "End If" после шага 25

                ' 7. Объявим переменную типа ObjectId и создадим её при помощи метода GetAt объекта ExtenstionDictionary из шага 5.
                ' Ипользуем строку "MyData" в качестве ключа entryName

                ' 8. Если мы попали сюда - значит данные уже внесены
                ' Воспользуемся методом WriteMessage объекта класса Editor (ed),
                ' созданного выше, для печати данных.
                ' В качестве аргумента выберем что-то типа:
                ' ControlChars.Lf + "This entity already has data..."

                ' 9. Теперь извлечем Xrecord. Объявим переменную типа Xrecord

                ' 10. Создадим переменную типа Xrecord используя
                ' метод GetObject объекта Transaction, созданного выше. (trans).
                ' В качестве аргумента ObjectId используем ObjectId, созданный на шаге 7
                ' Открываем для чтения

                ' 11. Здесь напечатаем значения из Xrecord в командную строку.
                ' Используем оператор "For Each". В качестве элемента используем TypedValue.
                ' (Используем value в качестве имени TypedValue)
                ' В качестве группы используем свойство Data объекта Xrecord
                ' Заметка: Поместим оператор Next после шага 12

                ' 12. Используем метод WriteMessage объекта Editor (ed).
                ' В качестве строки что-то подобное:
                ' ControlChars.Lf + value.TypeCode.ToString() + " . " + value.Value.ToString()

                ' 13. Если мы попали сюда - данных еще нет
                ' Метод UpgradeOpen() объекта ExtensionDictionary созданного
                ' на шаге 5, чтобы открыть на запись

                ' 14. Создаём новую XRecord. Объявим и создадим Xrecord при помощи New Xrecord

                ' 15. Создадим список resbuf. Объявим переменную типа ResultBuffer.
                ' Создадим её при помощи оператора New ResultBuffer.
                ' В качестве параметра используем следующее:
                ' New TypedValue(DxfCode.Int16, 1), _
                ' New TypedValue(DxfCode.Text, "MyStockData"), _
                ' New TypedValue(DxfCode.Real, 51.9), _
                ' New TypedValue(DxfCode.Real, 100.0), _
                ' New TypedValue(DxfCode.Real, 320.6)

                ' 16. Добавим ResultBuffer к Xrecord используя свойство Data
                ' обекта Xrecord. (делаем свойство равным ResultBuffer из шага 15)

                ' 17. Создадим запись в ExtensionDictionary. Воспользуемся методом SetAt объекта ExtensionDictionary из шага 5.
                ' В качестве аргумента SearchKey используем строку "MyData".
                ' В качестве аргумента DBObject  используем Xrecord, созданную
                ' на шаге 14.

                ' 18. Сообщим транзакции о новой Xrecord используя метод
                ' AddNewlyCreatedDBObject транзакции (trans)

                ' 19. Здесь мы можем заполнить контрол TreeView новыми данными
                ' Используем оператор "If Not Is Nothing Then" для того, чтобы
                ' проверить создана ли уже палитра (myPalette из Lab 4)
                ' (Если нет - будет ошибка)
                ' Заметка: Поместим "End If" после шага 25

                ' 20. Создадим оператор For Each. Используем имя node в качестве
                ' имени элемента и тип Forms.Treenode. В качестве группы -
                ' свойство Nodes переменной TreeView. (Me.myPalette.TreeView1.Nodes)
                ' Заметка: поместим Next после шага 25


                ' 21. Воспользуемся оператором "If Then". Проверяем равно ли
                ' свойство Tag объекта node Tag свойству ObjectId примитива ent
                ' с шага 1. Используем ObjectId. (ent.ObjectId.ToString)
                ' Заметка: поместим "End If" перед оператором "Next",
                ' добавленным на шаге 20

                ' 22. Теперь добавим новые данные к treenode. Объявим переменную
                ' типа Forms.Treenode. (назовем её childNode). Создадим её
                ' путем присвоения ей значения, возвращаемого методом Add
                ' свойства Nodes объекта node из цикла.
                ' (node.Nodes.Add) В качестве имени используем "Extension Dictionary"

                ' 23. Теперь добавим данные. Создаём оператор For Each.
                ' Имя элемента цикла - value, тип - TypedValue. Группа -
                ' свойство Data объекта Xrecord с шага 14
                ' Заметка: поместим оператор Next после шага 24

                ' 24. Добавим TypeValue из цикла For Each к TreeNode из шага 22.
                ' Используем метод Add коллекции Nodes.
                ' (childNode.Nodes.Add) В качестве строкового аргумента
                ' используем TypeValue из цикла. (value.ToString)


                ' 25. Выходим из цикла (всё сделано - покинем цикл)


                ' Всё прекрасно, завершим транзакцию

                trans.Commit()

            Catch ex As Exception
                ' если возникла проблема - сообщим о ней
                ed.WriteMessage("a problem occured because " + ex.Message)
            Finally
                ' В любом случае нужно удалить транзакцию
                trans.Dispose()

            End Try

        End If

    End Sub





    <CommandMethod("addDataToNOD")> _
    Public Sub addDataToNOD()

        ' получаем объект редактора
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' стартуем транзакцию
        Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
        Try
            ' 26. Здесь мы хотим добавить данные в Словарь именованных объектов.(NOD)
            ' Объявим переменную типа DBDictionary. (назовём её nod).
            ' Создадим её, присвоив ей значение, равное результату вызова
            ' метода GetObject объекта транзакции, созданного выше (trans).
            ' В качестве аргумента ObjectId используем свойство
            ' NamedObjectsDictionaryId текущей базы данных (Database):
            ' (ed.Document.Database.NamedObjectsDictionaryId)
            ' Открываем для чтения.

            ' 27. Нет ли еще записи, которую мы собираемся добавить, в словаре?
            ' Воспользуемся свойством Contains словаря для проверки в операторе
            ' "If Else End If" statement.
            ' Заметка: поместим "Else" после шага 33 и "End If" после шага 39

            ' 28. Объявим переменную типа ObjectId с именем entryId и создадим её
            ' используя метод GetAt объекта ExtenstionDictionary из шага 26.
            ' Используем строку "MyData" в качестве аргумента entryName.

            ' 29. Если мы попали сюда, значит в Словаре именованных данных уже есть наша информация
            ' Используем метод WriteMessage редактора. В качестве аргумента:
            ' ControlChars.Lf + "This entity already has data..."

            ' 30. Получим the Xrecord из NOD. Объявим переменную типа Xrecord

            ' 31. Используем транзакцию(trans) и её метод GetObject
            ' чтобы получить Xrecord из NOD. В качестве аргумента ObjectId
            ' используем ObjectId из шага 28. Открываем Xrecord для чтения

            ' 32. Выводим значения Xrecord в командную строку. Используем
            ' цикл "For Each". В качестве имени элемента используем value, тип
            ' TypedValue. Используем свойство Data объекта Xrecord из шага
            ' 31 в качестве аргумента группа
            ' Заметка: поместим the Next после шага 33

            ' 33. Используем метод WriteMessage редактора. В качестве сообщения:
            ' ControlChars.Lf + value.TypeCode.ToString() + " . " + value.Value.ToString()


            ' 34. Наши данные еще не в NOD, нужно их туда поместить.
            ' Переведём переменную NOD из шага 26 в состояние <для записи>

            ' 35. Объявим переменную типа Xrecord и создадим её.

            ' 36. Создадим список resbuf. Объявим переменную типа ResultBuffer. Создадим её при помощи New ResultBuffer. В качестве параметра:
            ' New TypedValue(DxfCode.Int16, 1), _
            ' New TypedValue(DxfCode.Text, "MyCompanyDefaultSettings"), _
            ' New TypedValue(DxfCode.Real, 51.9), _
            ' New TypedValue(DxfCode.Real, 100.0), _
            ' New TypedValue(DxfCode.Real, 320.6)


            ' 37. Добавим ResultBuffer к Xrecord используя его свойство Data
            ' (т.е. присваиваему ему значение ResultBuffer с шага 36)

            ' 38. Создаём запись в ExtensionDictionary. Используем метод SetAt
            ' Словаря именованных данных (NOD) из шага 26. В качестве SearchKey
            ' используем "MyData". В качестве DBObject используем Xrecord
            ' из шага 35.


            ' 39. Сообщим транзакции о новом объекте Xrecord используя метод
            ' AddNewlyCreatedDBObject транзакции (trans)

            trans.Commit()

        Catch ex As Exception
            ' есть проблема - сообщим о ней
            ed.WriteMessage("a problem occurred because " + ex.Message)
        Finally
            ' чтобы не случилось, а транзакцию освободи!
            trans.Dispose()

        End Try

    End Sub

End Class