﻿' (C) Copyright 2002-2005 by Autodesk, Inc.
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted,
' provided that the above copyright notice appears in all copies and
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports System.Windows

Imports Autodesk.AutoCAD.Windows

Public Class adskClass


    ' Определяем команду 'helloworld'
    <CommandMethod("helloworld")> _
    Public Sub helloworld()

        ' Получаем объект редактора
        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
        ' Пишем в командной строке
        ed.WriteMessage(vbCr + "Здравствуй, Мир!")

    End Sub

    <CommandMethod("addAnEnt")> _
    Public Sub AddAnEnt()

        ' Получаем объект редактора т.к. собираемся кое-что вводить
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' уточним какой тип примитива будем создавать
        Dim getWhichEntityOptions As PromptKeywordOptions = New PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block")
        ' теперь запросим его
        Dim getWhichEntityResult As PromptResult = ed.GetKeywords(getWhichEntityOptions)
        ' Если OK
        If (getWhichEntityResult.Status = PromptStatus.OK) Then

            ' Проверим какой тип примитива требуется создать
            Select Case getWhichEntityResult.StringResult

                Case "Circle"

                    ' Укажем центр окружности
                    Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick Center Point : ")
                    Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)
                    ' Если OK
                    If (getPointResult.Status = PromptStatus.OK) Then

                        ' получаем радиус
                        Dim getRadiusOptions As PromptDistanceOptions = New PromptDistanceOptions("Pick Radius : ")
                        ' в качестве базовой точки - точку которую мы указали для центра
                        getRadiusOptions.BasePoint = getPointResult.Value
                        ' сообщим механизму ввода использовать базовую точку
                        getRadiusOptions.UseBasePoint = True
                        ' теперь получим радиус
                        Dim getRadiusResult As PromptDoubleResult = ed.GetDistance(getRadiusOptions)
                        ' Если всё OK
                        If (getRadiusResult.Status = PromptStatus.OK) Then

                            ' Нужно добавить окружность к текущему пространству
                            ' Получаем активную базу
                            Dim dwg As Database = ed.Document.Database
                            ' стартуем транзакцию
                            Dim trans As Transaction = dwg.TransactionManager.StartTransaction()
                            Try

                                ' создаём новую окружность
                                Dim circle As New Circle(getPointResult.Value, Vector3d.ZAxis, getRadiusResult.Value)
                                ' открываем активное пространство (запись таблицы блоков) для записи
                                Dim btr As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                                ' добавляем окружность к текущему пространству (скорее всего пространство модели)
                                btr.AppendEntity(circle)
                                ' сообщим транзакции о необходимости автозакрыть объект окружности
                                trans.AddNewlyCreatedDBObject(circle, True)
                                ' завершаеи (commit) транзакцию
                                trans.Commit()

                            Catch ex As Exception
                                ' Есть исключение - сообщим о нём
                                ed.WriteMessage("problem due to " + ex.Message)
                            Finally
                                ' Дело сделано, в любом случае освободим транзакцию
                                trans.Dispose()
                            End Try

                        End If
                    End If

                Case "Block"

                    ' вводим имя блока
                    Dim blockNameOptions As PromptStringOptions = New PromptStringOptions("Enter name of the Block to create : ")
                    ' запрещаем пробелы в имени блока
                    blockNameOptions.AllowSpaces = False
                    ' получаем имя
                    Dim blockNameResult As PromptResult = ed.GetString(blockNameOptions)
                    ' Если ok
                    If (blockNameResult.Status = PromptStatus.OK) Then

                        ' Создадим определение блока
                        ' получаем активный чертеж
                        Dim dwg As Database = ed.Document.Database
                        ' начнем транзакцию
                        Dim trans As Transaction = dwg.TransactionManager.StartTransaction
                        Try

                            ' создаем новое определение блока
                            Dim newBlockDef As BlockTableRecord = New BlockTableRecord
                            ' задаём имя блоку
                            newBlockDef.Name = blockNameResult.StringResult

                            ' добавим определение блока в таблицу блоков
                            ' откроем таблицу блоков для чтения и проверим есть ли уже такое имя блока в ней
                            Dim blockTable As BlockTable = trans.GetObject(dwg.BlockTableId, OpenMode.ForRead)
                            ' проверяем есть ли уже такое имя
                            If (blockTable.Has(blockNameResult.StringResult) = False) Then
                                ' Если нет, то можем добавить его
                                ' только сначала нужно переключится в режим для записи
                                blockTable.UpgradeOpen()
                                blockTable.Add(newBlockDef)
                                ' сообщаем менеджеру транзакций о новом объекте для автозакрытия
                                trans.AddNewlyCreatedDBObject(newBlockDef, True)
                            End If

                            ' теперь добавим несколько объектов к определению блока
                            Dim circle1 As New Circle(New Point3d(0, 0, 0), Vector3d.ZAxis, 10)
                            Dim circle2 As New Circle(New Point3d(20, 10, 0), Vector3d.ZAxis, 10)
                            Dim circle3 As New Circle(New Point3d(40, 20, 0), Vector3d.ZAxis, 10)
                            Dim circle4 As New Circle(New Point3d(60, 30, 0), Vector3d.ZAxis, 10)
                            Dim circle5 As New Circle(New Point3d(80, 40, 0), Vector3d.ZAxis, 10)
                            newBlockDef.AppendEntity(circle1)
                            newBlockDef.AppendEntity(circle2)
                            newBlockDef.AppendEntity(circle3)
                            newBlockDef.AppendEntity(circle4)
                            newBlockDef.AppendEntity(circle5)
                            ' сообщим менеджеру транзакций о новых объектах
                            trans.AddNewlyCreatedDBObject(circle1, True)
                            trans.AddNewlyCreatedDBObject(circle2, True)
                            trans.AddNewlyCreatedDBObject(circle3, True)
                            trans.AddNewlyCreatedDBObject(circle4, True)
                            trans.AddNewlyCreatedDBObject(circle5, True)

                            ' теперь вставим блок в текущее пространство - запросим точку вставки
                            Dim blockRefPointOptions As PromptPointOptions = New PromptPointOptions("Pick insertion point of Block : ")
                            Dim blockRefPointResult As PromptPointResult = ed.GetPoint(blockRefPointOptions)
                            ' если не всё в порядке
                            If (blockRefPointResult.Status <> PromptStatus.OK) Then
                                ' освобождаем транзакцию, чтобы отменить изменения и выходим
                                trans.Dispose()
                                Return
                            End If
                            ' теперь у нас есть определение блока и точка для вставки - создадим вставку блока
                            Dim blockRef As BlockReference = New BlockReference(blockRefPointResult.Value, newBlockDef.ObjectId)
                            ' открываем текущее пространство для записи и добавим вставку блока в него
                            Dim curSpace As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                            ' добавляем вставку блока
                            curSpace.AppendEntity(blockRef)
                            ' укажем транзакции о новом объекте для автозакрытия
                            trans.AddNewlyCreatedDBObject(blockRef, True)

                            ' Если всё в норме - завершим транзакцию
                            trans.Commit()

                        Catch ex As Exception
                            ' Есть проблема - сообщим о ней
                            ed.WriteMessage("a problem occured because " + ex.Message)
                        Finally
                            ' чтобы не произошло, а транзакцию нужно освободить
                            trans.Dispose()

                        End Try

                    End If

            End Select
        End If

    End Sub

    ' Объявим объект PaletteSset - он должен быть создан только один раз
    Public myPaletteSet As Autodesk.AutoCAD.Windows.PaletteSet
    ' Нам нужна палитра, которая размещается на PaletteSet
    Public myPalette As UserControl1

    ' команда palette
    <CommandMethod("palette")> _
    Public Sub palette()

        ' проверим создана ли уже
        If (myPaletteSet = Nothing) Then
            ' создаём PaletteSet с уникальным GUID
            myPaletteSet = New Autodesk.AutoCAD.Windows.PaletteSet("My Palette", New Guid("D61D0875-A507-4b73-8B5F-9266BEACD596"))
            ' создаём новую палитру, внутри которой контрол дерево (Tree Control)
            myPalette = New UserControl1
            ' теперь добавим palette к paletteset
            myPaletteSet.Add("Palette1", myPalette)

        End If

        ' сделаем видимым paletteset
        myPaletteSet.Visible = True

    End Sub

    <CommandMethod("addDBEvents")> _
    Public Sub addDBEvents()

        ' нужно предварительно создать палитру
        If myPalette Is Nothing Then
            ' получаем объект редактора
            Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
            ' пишем в командной строке
            ed.WriteMessage(vbCr + "Please call the 'palette' command first")
            Exit Sub
        End If

        ' получаем активную базу
        Dim curDwg As Database = Application.DocumentManager.MdiActiveDocument.Database
        ' добавим обработчики событий
        AddHandler curDwg.ObjectAppended, New ObjectEventHandler(AddressOf callback_ObjectAppended)
        AddHandler curDwg.ObjectErased, New ObjectErasedEventHandler(AddressOf callback_ObjectErased)
        AddHandler curDwg.ObjectReappended, New ObjectEventHandler(AddressOf callback_ObjectReappended)
        AddHandler curDwg.ObjectUnappended, New ObjectEventHandler(AddressOf callback_ObjectUnappended)

    End Sub

    Private Sub callback_ObjectAppended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' добавим имя класса объекта к tree view
        Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
        ' записываем ID объекта для того, чтобы его можно было потом найти
        newNode.Tag = e.DBObject.ObjectId.ToString()

    End Sub

    Private Sub callback_ObjectErased(ByVal sender As Object, ByVal e As ObjectErasedEventArgs)

        ' Если объект удалён
        If e.Erased Then
            ' находим объект в контроле treeview чтобы удалить его
            For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
                ' если это он
                If (node.Tag = e.DBObject.ObjectId.ToString) Then
                    node.Remove()
                    Exit For
                End If
            Next
            ' если объект восстановлен
        Else
            ' добавим имя класса объекта к tree view
            Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
            ' записываем ID объекта для того, чтобы его можно было потом найти
            newNode.Tag = e.DBObject.ObjectId.ToString()
        End If
    End Sub
    Private Sub callback_ObjectReappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' добавим имя класса объекта к tree view
        Dim newNode As Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
        ' записываем ID объекта для того, чтобы его можно было потом найти
        newNode.Tag = e.DBObject.ObjectId.ToString()

    End Sub

    Private Sub callback_ObjectUnappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' находим объект в контроле treeview чтобы удалить его
        For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
            ' если это он
            If (node.Tag = e.DBObject.ObjectId.ToString) Then
                node.Remove()
                Exit For
            End If
        Next

    End Sub

    <CommandMethod("addData")> _
    Public Sub addData()

        ' получаем объект редактора
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ' указываем объект для добавления данных!
        Dim getEntityResult As PromptEntityResult = ed.GetEntity("Pick an entity to add an Extension Dictionary to : ")
        ' Если всё ok
        If (getEntityResult.Status = PromptStatus.OK) Then
            ' стартуем транзакцию
            Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
            Try

                ' открываем объект для чтения
                Dim ent As Entity = trans.GetObject(getEntityResult.ObjectId, OpenMode.ForRead)
                ' проверяем нет ли ещё ExtensionDictionary у этого примитива
                If (ent.ExtensionDictionary.IsNull) Then
                    ' Чтобы создать переходим в режим для записи
                    ent.UpgradeOpen()
                    ' создаём
                    ent.CreateExtensionDictionary()
                End If
                ' Теперь всё должно быть нормально
                ' Открываем для чтения
                Dim extensionDict As DBDictionary = trans.GetObject(ent.ExtensionDictionary, OpenMode.ForRead)

                Try
                    ' Проверяем есть у же наша запись - если нет, то возникнет исключение,
                    ' которое обработаем в Catch
                    Dim entryId As ObjectId = extensionDict.GetAt("MyData")

                    ' Если мы здесь, то всё OK
                    ' печатаем данные
                    ed.WriteMessage(ControlChars.Lf + "This entity already has data...")
                    ' извлекаем Xrecord
                    Dim myXrecord As Xrecord
                    ' читаем её из ExtensionDictionary
                    myXrecord = trans.GetObject(entryId, OpenMode.ForRead)
                    ' печатаем все её значения
                    For Each value As TypedValue In myXrecord.Data
                        ed.WriteMessage(ControlChars.Lf + value.TypeCode.ToString() + " . " + value.Value.ToString())
                    Next

                Catch
                    ' переходим в режим для записи
                    extensionDict.UpgradeOpen()

                    ' создаём новую Xrecord
                    Dim myXrecord As New Xrecord
                    ' создаём список буферов
                    Dim data As ResultBuffer = New ResultBuffer(New TypedValue(DxfCode.Int16, 1), _
                                                                New TypedValue(DxfCode.Text, "MyStockData"), _
                                                              New TypedValue(DxfCode.Real, 51.9), _
                                                              New TypedValue(DxfCode.Real, 100.0), _
                                                              New TypedValue(DxfCode.Real, 320.6))
                    ' добавляем его к Xrecord
                    myXrecord.Data = data

                    ' создаём запись
                    extensionDict.SetAt("MyData", myXrecord)
                    ' сообщаем транзакции о новом объекте Xrecord (для автозакрытия)
                    trans.AddNewlyCreatedDBObject(myXrecord, True)

                    ' теперь сообщим нашему TreeView контролу о новых данных
                    ' находим Tag, который нам нужен в TreeView
                    ' Неплохо бы проверить, что палитра уже создана - в противном случае будет аварийное завершение
                    If Not myPalette Is Nothing Then
                        For Each node As Forms.TreeNode In Me.myPalette.TreeView1.Nodes
                            ' Если это нужная нам запись
                            If (node.Tag = ent.ObjectId.ToString()) Then
                                ' добавим новую запись к записи (поддерево)
                                Dim childNode As Forms.TreeNode = node.Nodes.Add("Extension Dictionary")
                                ' теперь добавим данные
                                For Each value As TypedValue In myXrecord.Data
                                    childNode.Nodes.Add(value.ToString())
                                Next

                                ' Всё сделано - выходим из цикла
                                Exit For
                            End If
                        Next
                    End If
                End Try

                ' Всё сделано - завершаем транзакцию
                trans.Commit()

            Catch ex As Exception
                ' есть проблема - сообщим о ней
                ed.WriteMessage("a problem occured because " + ex.Message)
            Finally
                ' чтобы не случилось, а транзакцию нужно освободить
                trans.Dispose()

            End Try

        End If

    End Sub

    <CommandMethod("addDataToNOD")> _
    Public Sub addDataToNOD()

        ' получаем объект редактора
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ' стартуем транзакцию
        Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
        Try

            ' Открываем словарь именованных объектов (NOD) для чтения
            Dim nod As DBDictionary = trans.GetObject(ed.Document.Database.NamedObjectsDictionaryId, OpenMode.ForRead)

            Try
                ' проверяем есть ли наша запись уже в NOD - если нет, то возникнет исключение
                ' которое мы обработаем в Catch
                Dim entryId As ObjectId = nod.GetAt("MyData")

                ' Если мы здесь, то всё OK
                ' печатаем данные
                ed.WriteMessage(ControlChars.Lf + "This entity already has data...")
                ' извлекаем Xrecord
                Dim myXrecord As Xrecord
                ' читаем её из словаря NOD
                myXrecord = trans.GetObject(entryId, OpenMode.ForRead)
                ' печатаем все её значения
                For Each value As TypedValue In myXrecord.Data
                    ed.WriteMessage(ControlChars.Lf + value.TypeCode.ToString() + " . " + value.Value.ToString())
                Next

            Catch
                ' переключаемся в режим для записи
                nod.UpgradeOpen()

                ' создаём XRecord
                Dim myXrecord As New Xrecord
                ' создаём список буферов
                Dim data As ResultBuffer = New ResultBuffer(New TypedValue(DxfCode.Int16, 1), _
                                                            New TypedValue(DxfCode.Text, "MyCompanyDefaultSettings"), _
                                                            New TypedValue(DxfCode.Real, 51.9), _
                                                            New TypedValue(DxfCode.Real, 100.0), _
                                                            New TypedValue(DxfCode.Real, 320.6))
                ' добавляем его к Xrecord
                myXrecord.Data = data

                ' создаём запись
                nod.SetAt("MyData", myXrecord)
                ' сообщаем транзакции о новом объекте (для автозакрытия)
                trans.AddNewlyCreatedDBObject(myXrecord, True)

            End Try

            ' Всё OK, закрываем транзакцию
            trans.Commit()

        Catch ex As Exception
            ' есть проблема - сообщим о ней
            ed.WriteMessage("a problem occurred because " + ex.Message)
        Finally
            ' чтобы не случилось, а транзакцию нужно освободить
            trans.Dispose()

        End Try

    End Sub



    ' Начало Lab6
    ' Заметка: На шаге 10 вы добавите tooltip в PointMonitor.
    ' В AutoCAD 2012 есть проблема, что tooltip невидна, если ROLLOVERTIPS = 0
    ' Это должно быть исправлено в Service Pack 1.
    ' Так же в AutoCAD 2012 вы можете получить сообщение,
    ' когда наводите курсор на примитивы после добавления PointMonitor.
    ' (просто проигнорируйте его) Эта проблема передана программистам AutoCAD.

    ' 1. Используем атрибут CommandMethod и создаём команду с именем "addpointmonitor"
    ' Имя процедуры (Sub) startMonitor
    ' Заметка: Поместите "End Sub" после шага 3

    ' 2. Объявим переменную типа Editor с именем ed. Создадим её используя свойство Editor
    ' объекта Application.DocumentManager.MdiActiveDocument

    ' 3. Используем оператор AddHandler для обработки события редактора PointMonitor
    ' Для этого используем событие PointMonitor переменной ed
    ' созданной на шаге 2 в качестве первого параметра (Event).
    ' В качестве второго параметра (Delegate) используем оператор New
    ' для класса PointMonitorEventHandler, используя оператор AddressOf
    ' и имя процедуры (MyPointMonitor) Процедура будет создана на шаге 4.

    ' 4. Создаем Public Sub с именем MyPointMonitor. Эта процедура будет вызваться при каждом движении мыши
    ' (имя должно совпадать с тем, которое у параметра Delegate с шага 3).
    ' Первый параметр процедуры - объект. (Используем ByVal и имя sender).
    ' Второй параметр PointMonitorEventArgs. (Используем ByVal и имя e)
    ' Заметка: Поместим End Sub после шага 27

    ' 5. Объявим массив бъектов FullSubentityPath. В качестве имени
    ' массива используем fullEntPath. Создаём его путём присвоения
    ' ему результата выполнения метода GetPickedEntities свойства Context
    ' объекта PointMonitorEventArgs переданного в Sub

    ' 6. Используем оператор "If Then" и проверим свойство Length массива
    ' созданного на шаге 5
    ' Заметка: Поместим "End If" после шага 27

    ' 7. Объявим переменную с именем trans типа Transaction.
    ' Создаём её делая равной результату выполнения метода
    ' StartTransaction объекта TransactionManager
    ' текущей базы данных.
    ' Application.DocumentManager.MdiActiveDocument.Database.TransactionManager.StartTransaction

    ' 8. Создаём блок Try/Catch/Finally.
    ' Заметка: Поместим Catch после шага 25 и оператор Finally после шага 26
    ' Поместим End Try после шага 27

    ' 9. Объявим переменную типа Entity. Создадим её используя
    ' метод GetObject транзакции из шага 7. В качестве параметра
    ' ObjectId используем первый элемент в GetObjectIds(0)
    ' нулевой элемент в массиве FullSubentityPath созданном
    ' на шаге 5. Открываем примитив для чтения

    ' 10. Добавим подсказку (tooltip) используя метод
    ' AppendToolTipText объекта PointMonitorEventArgs, переданного в Sub.
    ' Используем в качестве строки подсказки:
    ' "The Entity is a " + ent.GetType().ToString()

    ' 11. Используем оператор "If Then" и проверим создана ли палитра (myPalette)
    ' Если нет - то выход.

    ' 12. Следующие шаги делают текст для DBEntity
    ' в палитре созданной в Lab4 полужирным (Bold). Объявим
    ' переменную с именем fontRegular класса System.Drawing.Font.
    ' Создаём её присвоением ей New System.Drawing.Font.
    ' В качестве аргументов:
    ' "Microsoft Sans Serif", 8, Drawing.FontStyle.Regular

    ' 13. Объявим переменную с именем fontBold типа System.Drawing.Font.
    ' Создаём её присвоением ей New System.Drawing.Font.
    ' В качестве аргументов:
    ' "Microsoft Sans Serif", 8, Drawing.FontStyle.Bold


    ' 14. Используем метод SuspendLayout() объекта TreeView,
    ' созданного в Lab4 чтобы подождать пока следующие шаги будут выполнены.
    ' Используем ключевое слово "Me" чтобы получить палитру
    ' (myPalette) с the TreeView.

    ' 15. Здесь мы найдём объект в контроле treeview, чтобы изменить
    ' шрифт на полужирный (bold).
    ' Создадим цикл For Each. Переменная цикла с именем node типа
    ' Forms.Treenode. Параметр группа - свойство Nodes объекта TreeView.
    ' (myPalette.TreeView1.Nodes)
    ' Заметка: поместим оператор Next после шага 21.
    ' (В этом цикле, если курсор над примитивом, для
    ' которого есть запись в TreeView, запись будет подсвечена).

    ' 16. Используем оператор "If Then Else" и смотрим равно ли
    ' свойство Tag объекта node значению свойства ObjectId примитива
    ' объявленого на шаге 9.
    ' (Тспользуем ToString для сравнения)
    ' Заметка: Поместим "Else" после шага 19
    ' Поместим "End IF" после шага 21.

    ' 17. Если мы здесь, значит мы нашли node, который нас интересует.
    ' Используем оператор "If Not" и метод Equals переменной типа
    ' System.Drawing.Font созданной на шаге 12. В качестве параметра
    ' Object используем свойство NodeFont объекта node.
    ' Заметка: Поместим "End If" после шага 19

    ' 18. Присвоим свойству NodeFont обекта node значение переменной
    ' System.Drawing.Font созданной на шаге 13.

    ' 19. Сделаем свойство Text объекта node равным свойству
    ' node.Text.

    ' 20. Если мы попали сюда, значит это не тот node, который нам нужен.
    ' Используем оператор "If Not" и метод Equals переменной
    ' System.Drawing.Font созданной на шаге 12. В качестве параметра
    ' Object используем свойство NodeFont объекта node.
    ' Заметка: Поместим "End If" после шага 21.


    ' 21. Сделаем свойство NodeFont объекта node равным
    ' переменной System.Drawing.Font созданной на шаге 12


    ' 22. Теперь самое время выполнить пересчет TreeView. Воспользуемся
    ' методом ResumeLayout() обекта TreeView. Используем ключевое слово "Me" для получения палитры с TreeView.

    ' 23. Перерисуем TreView при помощи метода Refresh объекта TreeView.

    ' 24. Обновим TreView при помощи метода Update объекта TreeView.

    ' 25. Всё нормально если мы попали сюда, так что завершим (Commit) транзакцию, созданную на шаге 7.

    ' 26. Используем WriteMessage объекта типа Editor, чтобы
    ' напечатать сообщение исключения в командной строке AutoCAD
    ' Application.DocumentManager.MdiActiveDocument.Editor


    ' 27. Чтобы не случилось, а удалить (Dispose) транзакцию мы обязаны.
    ' (Это в блоке Finally). Используем метод Dispose транзакции,
    ' созданной на шаге 7.
    ' Заметка: продолжим на шаге 28 в функции NewInput.
    ' Вы также должны создать и проверить команду addPointmonitor
    ' перед тем, как завершить следующие шаги.


    <CommandMethod("newInput")> _
    Public Sub NewInput()

        ' Запустим наш InputPointMonitor
        ' Получаем объект редактора
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ' теперь добавим делегат к списку событий
        AddHandler ed.PointMonitor, New PointMonitorEventHandler(AddressOf MyInputMonitor)

        ' 28. Необходимо включить механизм AutoCAD обеспечивающий обработку всех объектов внутри апертуры
        ' для всех событий мыши, не зависимо от того была ли точка указана и включены ли режимы привязки
        ' Воспользуемся для этого методом TurnForcedPickOn
        ' объекта Editor созданного выше "ed"


        ' 29. Здесь мы попросим пользователя указать точку. Объявим переменную типа
        ' PromptPointOptions. Создадим её оператором New PromptPointOptions
        ' в качестве параметра Message используем строку "Pick A Point : "

        ' 30. Объявим переменную типа PromptPointResult. Создадим её
        ' используя метод GetPoint объекта редактора (ed)
        ' Передадим в качестве параметра объект типа PromptPointOptions
        ' созданный на шаге 29

        ' Если всё в порядке:
        'If (getPointResult.Status = PromptStatus.OK) Then
        '    ' что-то делаем...
        'End If

        ' 31. Теперь удалим наш InputPointMonitor, т.к. мы закончили с ним.
        ' Используем оператор RemoveHandler для объекта, используя
        ' свойство PointMonitor объекта редактора (ed)
        ' В качестве объекта Delegate используем AddressOf и
        ' имя функции обратного вызова "MyInputMonitor"
        ' Продолжим с шага 31 в процедуре MyInputMonitor.

    End Sub

    Public Sub MyInputMonitor(ByVal sender As Object, ByVal e As PointMonitorEventArgs)

        ' Сначала проверим, что у нас под курсором
        Dim fullEntPath() As FullSubentityPath = e.Context.GetPickedEntities()
        If (fullEntPath.Length) Then

            ' стартуем транзакцию
            Dim trans As Transaction = Application.DocumentManager.MdiActiveDocument.Database.TransactionManager.StartTransaction
            Try

                ' открываем Entity для чтения, он должен быть наследником Curve
                Dim ent As Curve = trans.GetObject(fullEntPath(0).GetObjectIds(0), OpenMode.ForRead)

                ' Курсор действительно над кривой, проверим
                ' есть ли у неё ExtensionDictionary
                If (ent.ExtensionDictionary.IsValid) Then

                    ' Открываем ExtensionDictionary для чтения
                    Dim extensionDict As DBDictionary = trans.GetObject(ent.ExtensionDictionary, OpenMode.ForRead)

                    ' Находим в ней запись с ключом
                    Dim entryId As ObjectId = extensionDict.GetAt("MyData")

                    ' Если мы здесь, значит всё Ok
                    ' Извлекаем xrecord
                    Dim myXrecord As Xrecord
                    ' Читаем её из ExtensionDictionary
                    myXrecord = trans.GetObject(entryId, OpenMode.ForRead)


                    ' 32. ' Мы можем нарисовать временную графику в нескольких точках вдоль примитива
                    ' Создаём цикл "For Each". В качестве эдемента используем тип TypedValue с именем
                    ' myTypeVal. В качестве группы - свойство Data объекта Xrecord
                    ' созданного выше (myXrecord).
                    ' Заметка: поместим "Next" после шага 45.

                    ' 33. Используем оператор "If Then" и смотрим действительно ли TypeCode в the TypedValue
                    ' плавающий. (Используем DxfCode.Real для проверки).
                    ' Заметка: Поместим "End If" после шага 45.


                    ' 34. Для размещения временной графики вдоль кривой мы должны получить
                    ' точки вдоль кривой.
                    ' Объявим переменную класса Point3d. Создадим её используя метод
                    ' GetPointAtDist объекта ent
                    ' В качестве параметра Value используем значение свойства Value объекта TypedValue

                    ' 35. Нам для работы нужно узнать сколько пикселей в единичном квадрате
                    ' чтобы наша временная графика не зависела от масштаба изображения
                    ' Объявим переменную типа Point2d с именем
                    ' pixels. Создадим её используя метод GetNumPixelsInUnitSquare
                    ' текущего Viewport. (Передадим в него Point3d созданный на шаге 34).
                    ' Используем: e.Context.DrawContext.Viewport.GetNumPixelsInUnitSquare()


                    ' 36. Нам нужно несколько постоянных расстояний. Объявим переменнйю типа Double
                    ' с именем xDist. Сделаем её равной частному от деления 10 на
                    ' свойство X переменной типа Point2d из шага 35.

                    ' 37. Объявим переменную типа Double с именем yDist.
                    ' присвоим ей значение равное частному от деления 10 на
                    ' свойство Y переменной типа Point2d из шага 35


                    ' 38. Рисуем временную графику. Объявим переменную типа Circle
                    ' Создаём её оператором New Circle. Используем переменную типа Point3d
                    ' созданную на шаге 33 в качестве центра. В качестве нормали используем
                    ' Vector3d.ZAxis. В качестве радиуса используем плавающее число из шага 36.

                    ' 39. Используем метод Draw, чтобы показать окружность. (передаём Circle).
                    ' Используем:  e.Context.DrawContext.Geometry.Draw()


                    ' 40. Здесь мы добавим еще временную графику (текст). Объявим переменную типа
                    ' DBText. Создаём её при помощи оператора New DBText.

                    ' 41. Всегда хорошая идея устанавливать новому примитиву значения
                    ' по-умолчанию (текущие) для базы (это слой, тип
                    ' линии, текстовый стиль и т.д.)
                    ' Используем метод SetDatabaseDefaults объекта DBText
                    ' из шага 40

                    ' 42. Установим точку вставки текста, совпадающую с центром окружности.
                    ' но смещенную на радиус. Используем свойство Position и приравниваем его
                    ' Point3d из шага 34 плюс New Vector3d. В качестве
                    ' параметра X используем плавающее из шага 36, В качестве параметра Y
                    ' используем Double с шага 37. Для Z используем 0.

                    ' 43. Используем данные из Xrecord в качестве текста. Используем свойство TextString
                    ' объекта DBText созданного на шаге 49. Приравниваем его значению
                    ' Value объекта TypedValue. (use ToString)

                    ' 44. Высоту текста (Height) делаем равным значению Double созданному на шаге 36

                    ' 45. Используем метод Draw для отрисовки текста. (Передаём ему DBText).
                    ' Используем:  e.Context.DrawContext.Geometry.Draw()
                    ' Заметка: Цвет фона может влиять на отображение временной графики текста
                    ' (он отображается белым в этом примере, так что возможно
                    ' потребуется изменить цвет фона или изменить цвет DBText)

                    ' Конец Lab6


                End If

                ' Всё нормально, завершим транзакцию
                trans.Commit()
            Catch ex As Exception

            Finally
                ' Чтобы не случилось, а транзакцию нужно удалить
                trans.Dispose()

            End Try

        End If

    End Sub


End Class