﻿// (C) Copyright 2010 by Autodesk, Inc. 
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;

// This line is not mandatory, but improves loading performances
[assembly: CommandClass(typeof(Lab2_Complete.MyCommands))]

namespace Lab2_Complete
{

    // Экземпляры этого класса создаются для каждого документа,
    // в котором в первый раз выполняется команда в контексте данного документа
    // Другими словами, не статические данные для каждого документа свои!

    public class MyCommands
    {
        // Атрибут CommandMethod можно применять к любому публичной функции-члену любого публичного класса
        // Эта функция не должна иметь аргументов и не должна ничего возвращать.
        // Если метод - метод экземпляра класса, то его экземпляры классы класса создаются 
        // для каждого документа. Если метод - статический член класса, то экземпляры не создаются
        //
        // Заметка: CommandMethod имеет перегруженные варианты с helpid и контекстным меню


        // Модальная команда с локализованным именем
        [CommandMethod("MyGroup", "MyCommand", "MyCommandLocal", CommandFlags.Modal)]
        public void MyCommand() // Этот метод может иметь любое имя
        {
            // Поместите свой код сюда
        }

        // Модальная команда с предварительным выбором (pickfirst) selection
        [CommandMethod("MyGroup", "MyPickFirst", "MyPickFirstLocal", CommandFlags.Modal | CommandFlags.UsePickSet)]
        public void MyPickFirst() // Этот метод может иметь любое имя
        {
            PromptSelectionResult result = Application.DocumentManager.MdiActiveDocument.Editor.GetSelection();
            if (result.Status == PromptStatus.OK)
            {
                // Есть выбранные объекты
                // Поместите сюда код использующий предварительный выбор
            }
            else
            {
                // Нет выбранных объектов
                // Поместите сюда код 
            }
        }

        // Команда выполняющаяся в контексте приложения с локализованным именем
        [CommandMethod("MyGroup", "MySessionCmd", "MySessionCmdLocal", CommandFlags.Modal | CommandFlags.Session)]
        public void MySessionCmd() // Этот метод может иметь любое имя
        {
            // Поместите сюда код 
        }

        // Начало Lab2 
        // 1. Добавим команду с именем addAnEnt. Используем атрибут CommandMethod и
        // Public void функцию. 
        // Заметка: поместите "}" сразу после шага 21.
        [CommandMethod("addAnEnt")]
        public void AddAnEnt()
        {

            // 2. Объявим переменную типа Editor с именем ed. Создадим её с помощью свойства Editor документа 
            // Application.DocumentManager.MdiActiveDocument 
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

            // 3. Объявим переменную типа PromptKeywordOptions и создаём её оператором 
            // new PromptKeywordOptions. Используем строку для параметров message и Keywords
            // "Which entity do you want to create? [Circle/Block] : ", "Circle Block" 
            PromptKeywordOptions getWhichEntityOptions = new PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block");

            // 4. Объявим переменную типа PromptResult. Используем метод GetKeywords переменной типа Editor 
            // с шага 1. Передаём в него PromptKeywordOptions с шага 2. Создаём 
            // PromptResult приравняв его результату метода GetKeywords. 
            PromptResult getWhichEntityResult = ed.GetKeywords(getWhichEntityOptions);

            // 5. Добавим оператор if для проверки свойства Status переменной шага PromptResult с шага 4. 
            // Используем перечисление PromptStatus для проверки. (проверяем на равенство с PromptStatus.OK) 
            // Заметка: поместите "}" сразу после шага 21.
            // (после следующих инструкций) 
            if (getWhichEntityResult.Status == PromptStatus.OK)
            {

                // 6. Status равен PromptStatus.OK. Теперь запишем оператор switch. В качестве аргумента
                // используем свойство StringResult переменной PromptResult
                // Заметка: поместите "}" сразу после шага 21.
                // (После "}" оператора if с шага 5)
                switch (getWhichEntityResult.StringResult)
                {

                    // 7. Используем "Circle" для оператора case. (Если StringResult равен "Circle") Ниже 
                    // мы используем "Block" для case. (перепрыгнем к шагу 15 и добавим оператор break 
                    // чтобы избавится от сообщения "Control cannot fall through... ") 
                    case "Circle":

                        // 8. Мы хотим спросить у пользователя центр окружности. Объявим 
                        // переменную типа PromptPointOptions и создадим её оператором 
                        // new PromptPointOptions. Используем "Pick Center Point : " в качестве параметра message 

                        PromptPointOptions getPointOptions = new PromptPointOptions("Pick Center Point : ");

                        // 9. Объявим переменную типа PromptPointResult. Используем метод GetPoint 
                        // объекта Editor с шага 2. (параметр - PromptPointOptions с шага 8)
                        // Создадим PromptPointResult присвоением результата вызова метода 
                        // GetPoint. 
                        PromptPointResult getPointResult = ed.GetPoint(getPointOptions);

                        // 10. Добавим оператор if, который проверит свойство Status объекта PromptPointResult 
                        // с шага 9. Используем перечисление PromptStatus для проверки. (чтобы быть уверенным, что всё OK) 
                        // Заметка: поместите "}" сразу после шага 15.
                        if ((getPointResult.Status == PromptStatus.OK))
                        {

                            // 11. Теперь мы хотим спросить у пользователя радиус окружности. Объявим 
                            // переменную типа PromptDistanceOptions. Создадим её оператором a new PromptDistanceOptions. 
                            // Используем "Pick Radius : " в качестве параметра message. 
                            PromptDistanceOptions getRadiusOptions = new PromptDistanceOptions("Pick Radius : ");

                            // 12. Мы хотим использовать точку, выбранную на шаге 9 как 
                            // базовую точку для метода GetDistance. Для этого установим свойство 
                            // BasePoint переменной типа PromptDistanceOptions с предыдущего шага 
                            // Присваиваем ей значение свойства Value переменной типа
                            // PromptPointResult созданной на шаге 9. 
                            getRadiusOptions.BasePoint = getPointResult.Value;


                            // 13. Нам нужно сообщить механизму ввода, что нужно действительно использовать базовую точку 
                            // Чтобы это сделать, присвоим свойству UseBasePoint переменной типа 
                            // PromptDistanceOptions с шага 11 значение true. 
                            getRadiusOptions.UseBasePoint = true;

                            // 14. Получаем радиус окружности. Объявим переменную типа PromptDoubleResult. 
                            // Присваиваем ей результат выполнения метода GetDistance объекта Editor с шага 2 
                            // Передаём параметр – переменную типа PromptDistanceOptions с шага 11 
                            // и измененную на предыдущих шагах. 
                            PromptDoubleResult getRadiusResult = ed.GetDistance(getRadiusOptions);
                        }
                        // 15. Добавим оператор break, чтобы обозначить конец кода для "Circle". 
                        break;
                        // 16. Добавим оператор case для "Block" (прыгнем на шаг 20 и добавим оператор break 
                        // чтобы избавится от сообщения "Control cannot fall through... ") 
                    case "Block":

                        // 17. Здесь мы хотим спросить пользователя имя блока. Объявим переменную типа
                        // PromptStringOptions и создадим её оператором new 
                        // PromptStringOptions. Используем "Enter name of the Block to create : " for 
                        // в качестве параметра message. 
                        PromptStringOptions blockNameOptions = new PromptStringOptions("Enter name of the Block to create : ");

                        // 18. Мы не хотим, чтобы в имени блока были пробелы, так что запретим их. 
                        // Для этого установим свойство AllowSpaces объекта PromptStringOptions с шага 15 в false. 

                        blockNameOptions.AllowSpaces = false;

                        // 19. Получим имя, которое введёт пользователь. Объявим переменную типа PromptResult 
                        // и присвоим ей значение, возвращаемое методом GetString объекта типа Editor 
                        // с шага 2. Передадим как параметр PromptStringOptions с шага 17. 
                        PromptResult blockNameResult = ed.GetString(blockNameOptions);

                        // 20. Добавим оператор break, чтобы отметить конец обработки кода для case "Block" 
                        break;

                        // 21. Построим (Build) проект. Вставим точку останова. Воспользуемся командой NETLOAD 
                        // и запустим команду AddAnEnt. Пошагово выполним код и исправим ошибки. 
                        // Не забудьте запустить команду и протестировать код и для круга и для блока. 
                        // Конец Lab2
                }

            }
        }


        // Атрибут LispFunction подобен CommandMethod но создает функцию, которую можно вызвать
        // из lisp. Множество возвращаемых типов, а не только строки или целые числа.
        [LispFunction("MyLispFunction", "MyLispFunctionLocal")]
        public int MyLispFunction(ResultBuffer args)  // Этот метод может иметь любое имя
        {
            // Поместите свой код сюда

            // Возвращаем значение в интерпретатор Lisp
            return 1;
        }
    }

}
