﻿' (C) Copyright 2010 by Autodesk, Inc. 
'
Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

' Эта строка не обязательна, но ускоряет загрузку
<Assembly: CommandClass(GetType(Lab2_Complete.MyCommands))> 

Namespace Lab2_Complete

    ' Этот класс создаётся AutoCAD для каждого документа, когда
    ' пользователь первый раз вызывает команду в контексте документа
    ' Другими словами нет статических данных разделяемых между документами

    Public Class MyCommands

        ' Атрибут CommandMethod можно применять к любому публичному (Public) методу 
        ' любого публичного класса.
        ' Он не должен принимать параметры и не должен ничего возвращать.
        ' Если метод - метод экземпляра, то экземпляр класса, который его содержит, создаётся для каждого документа
        ' в котором он вызывается. Если это статический член класса, то экземпляры этого класса НЕ создаются.
        '
        ' Заметка: CommandMethod имеет переопределенные методы, которые могут обеспечить helpid и
        ' контекстное меню.

        ' Модальная команда с локализованным именем.
        ' AutoCAD выполняет поиск ресурсной строки с именем "MyCommandLocal" в том же самом 
        ' пространстве имён, в котором содержится класс. 
        ' Если строка ресурса не найдена, тогда строка "MyLocalCommand" используется 
        ' в качестве локализованного имени команды.
        ' Чтобы смотреть/редактировать resx-файл с ресурсной строкой для этой команды, 
        ' * нажмите кнопку 'Show All Files' в Solution Explorer (должен быть выбран файл myCommands.vb);
        ' * разверните узел дерева для файла myCommands.vb;
        ' * и сделайте двойной клик на myCommands.resx
        <CommandMethod("MyGroup", "MyCommand", "MyCommandLocal", CommandFlags.Modal)> _
        Public Sub MyCommand() ' Этот метод может иметь любое имя
            ' Добавьте сюда код
        End Sub

        ' Модальная команда с предварительным выбором (pickfirst selection)
        <CommandMethod("MyGroup", "MyPickFirst", "MyPickFirstLocal", CommandFlags.Modal + CommandFlags.UsePickSet)> _
        Public Sub MyPickFirst() ' Этот метод может иметь любое имя
            Dim result As PromptSelectionResult = Application.DocumentManager.MdiActiveDocument.Editor.GetSelection()
            If (result.Status = PromptStatus.OK) Then
                ' Здесь выбраны примитивы
                ' Поместите сюда свою команду, использующую pickfirst
            Else
                ' Нет выбраных примитивов
                ' Поместите свою команду сюда
            End If
        End Sub

        ' Команда работающая в контексте приложения с локализованым именем
        <CommandMethod("MyGroup", "MySessionCmd", "MySessionCmdLocal", CommandFlags.Modal + CommandFlags.Session)> _
        Public Sub MySessionCmd() ' Этот метод может иметь любое имя
            ' Поместите код своей команды сюда
        End Sub


        ' Начало Lab2

        ' 1. Добавим команду с именем addAnEnt. Воспользуемся атрибутом  CommandMethod и 
        ' Public Sub. (используем символ продолжения строки " _")
        ' Заметка: Поместите End Sub после шага 19.
        <CommandMethod("addAnEnt")> _
        Public Sub AddAnEnt()

            ' 2. Объявим переменную типа Editor с именем ed. Присвоим ей значение равное
            ' свойству Editor объекта Application.DocumentManager.MdiActiveDocument
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            ' 3. Объявим переменную типа PromptKeywordOptions и присвоим ей значение при помощи
            ' New PromptKeywordOptions. Используем строку подобную следующей для
            ' строки параметров messageAndKeywords.
            ' "Which entity do you want to create? [Circle/Block] : ", "Circle Block"
            Dim getWhichEntityOptions As PromptKeywordOptions = New PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block")

            ' 4. Объявим переменную типа PromptResult. Воспользуемся методом GetKeywords переменной ed, созданной на шаге 1
            ' Передадим методу параметр типа PromptKeywordOptions созданный на шаге 2.
            ' Присвоим переменной типа PromptResult значение, которое возвращает метод GetKeywords.
            Dim getWhichEntityResult As PromptResult = ed.GetKeywords(getWhichEntityOptions)

            ' 5. Добавим оператор If Then для проверки свойства Status переменной типа PromptResult
            ' созданного на шаге 4. Воспользуемся перечислением PromptStatus для этой проверки.
            ' (смотрим равен ли он PromptStatus.OK) 
            ' Заметка. Перенесем оператор End If после шага 19. 
            If (getWhichEntityResult.Status = PromptStatus.OK) Then

                ' 6. PromptStatus равен OK. Теперь запишем оператор Select Case. В качестве проверяемого значения используем
                ' Свойство StringResult переменной класса PromptResult используемой выше
                ' Заметка: Перенесите оператор End Select после шага 19.
                ' (Перед оператором End If)
                Select Case getWhichEntityResult.StringResult

                    ' 7. Используем "Circle" в качестве варианта. (Если StringResult равно "Circle")
                    ' (На шаге 15 мы добавим вариант для "Block").
                    Case "Circle"

                        ' 8. Мы хотим запросить у пользователя центр окружности. Объявим
                        ' переменную типа PromptPointOptions и присвоим ей значение равное
                        ' New PromptPointOptions. Используем строку "Pick Center Point : " в 
                        ' качестве параметра message 
                        Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick Center Point : ")

                        ' 9. Объявим переменную типа PromptPointResult. 
                        ' Воспользуемся методом GetPoint объекта ed класса Editor и шага 2
                        ' Передадим переменную PromptPointOptions созданную на шаге 8
                        ' Присвоим переменной PromptPointResult значение,
                        ' которое возвращает метод GetPoint method.
                        Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)

                        ' 10. Добавим оператор If Then для проверки свойства Status переменной типа PromptPointResult 
                        ' созданной на шаге 9. Используем перечисление PromptStatus для проверки. 
                        ' (убедимся, что всё в порядке)
                        ' Заметка: Перенесём оператор End If прямо перед шагом 15. 
                        If (getPointResult.Status = PromptStatus.OK) Then

                            ' 11. Теперь мы можем спросить у пользователя радиус окружности
                            ' Объявим переменную типа  PromptDistanceOptions. Присвоим ей значение равное
                            ' New PromptDistanceOptions. 
                            ' Используем строку "Pick Radius : " в качестве параметра message. 
                            Dim getRadiusOptions As PromptDistanceOptions = New PromptDistanceOptions("Pick Radius : ")

                            ' 12.  Мы хотим воспользоваться точкой, полученной на шаге 9 
                            ' в качестве базовой точки для метода GetDistance. Для этого воспользуемся
                            ' свойством BasePoint переменной класса PromptDistanceOptions,
                            ' созданной на предыдущем шаге. Присвоим свойству BasePoint значение равное свойству Value
                            ' переменной PromptPointResult созданной на шаге 9. 
                            getRadiusOptions.BasePoint = getPointResult.Value


                            ' 13. Нам нужно сообщить механизму ввода, что действительно нужно использовать базовую точку.
                            ' Чтобы сделать это, установим свойство UseBasePoint переменной типа 
                            ' PromptDistanceOptions, созданной на шаге 11, в True. 
                            getRadiusOptions.UseBasePoint = True

                            ' 14. Получим радиус окружности. Объявим переменную класса PromptDoubleResult.
                            ' Присвоим ей значение равное результату вызова метода GetDistance переменной типа Editor, созданной на шаге 2
                            ' Передадим переменную PromptDistanceOptions созданной на шаге 11 и
                            ' модифицированную на предыдущем шаге. 

                            Dim getRadiusResult As PromptDoubleResult = ed.GetDistance(getRadiusOptions)
                        End If
                        ' 15. Добавим инструкцию Case для строки "Block"
                    Case "Block"

                        ' 16. Теперь мы хотим спросить у пользователя имя блока. Объявим
                        ' переменную типа PromptStringOptions и присвоим ей значение
                        ' New PromptStringOptions. Используем строку "Enter name of the Block to create : "
                        ' в качестве параметра message.

                        Dim blockNameOptions As PromptStringOptions = New PromptStringOptions("Enter name of the Block to create : ")

                        ' 17. Пробелы недопустимы в качестве имени блока, так что запретим их.
                        ' Для этого установим свойство AllowSpaces переменной типа
                        ' PromptStringOptions, созданной на шаге 16 в False.
                        blockNameOptions.AllowSpaces = False

                        ' 18. Получим указанное пользователем имя. Объявим переменную типа PromptResult
                        ' и присвоим ей значение, которое возвращает метод GetString объекта ed класса Editor
                        ' созданной на шаге 2. Передадим значение переменной типа PromptStringOptions созданной на шаге 16.
                        Dim blockNameResult As PromptResult = ed.GetString(blockNameOptions)

                        ' 19. Построим проект (Build). Установим точку остановки (break point).
                        ' Воспользуемся командой NETLOAD и выполним команду AddAnEnt.
                        ' Пошагово выполним код и исправим ошибки.
                        ' Не забудьте запустить команду и протестировать код и для круга и для блока.
                        ' Конец Lab2
                End Select
            End If

        End Sub

        ' Атрибут LispFunction подобен CommandMethod, но создаёт функцию, 
        ' которую можно вызвать из Lisp
        ' Множество разных возвращаемых типов, не только строка или целое
        <LispFunction("MyLispFunction", "MyLispFunctionLocal")> _
        Public Function MyLispFunction(ByVal args As ResultBuffer) ' Этот метод может иметь любое имя
            ' Поместите код здесь

            ' Возвращаем значение в интерпретатор Lisp
            Return 1
        End Function

    End Class

End Namespace