﻿
' (C) Copyright 2002-2005 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted, 
' provided that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting 
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry


Public Class adskClass

    <CommandMethod("addAnEnt")> _
    Public Sub AddAnEnt()

        ' получаем объект редактора чтобы обеспечить пользовательский ввод
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' определимся какой тип примитива мы хотим создать
        Dim getWhichEntityOptions As PromptKeywordOptions = New PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block")
        ' теперь запрос к пользователю
        Dim getWhichEntityResult As PromptResult = ed.GetKeywords(getWhichEntityOptions)
        ' Если всё OK
        If (getWhichEntityResult.Status = PromptStatus.OK) Then

            ' Проверяем что выбрал пользователь
            Select Case getWhichEntityResult.StringResult

                Case "Circle"

                    ' Указываем центр окружности
                    Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick Center Point : ")
                    Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)
                    ' Если OK

                    If (getPointResult.Status = PromptStatus.OK) Then

                        ' Получаем радиус
                        Dim getRadiusOptions As PromptDistanceOptions = New PromptDistanceOptions("Pick Radius : ")
                        ' устанавливаем базовую точку (которую мы только что указали)
                        getRadiusOptions.BasePoint = getPointResult.Value
                        ' указываем механизму ввода действительно использовать базовую точку
                        getRadiusOptions.UseBasePoint = True
                        ' получаем радиус
                        Dim getRadiusResult As PromptDoubleResult = ed.GetDistance(getRadiusOptions)
                        ' Если всё OK

                        If (getRadiusResult.Status = PromptStatus.OK) Then

                            ' Начало Lab3. Создаём окружность (Circle) или блок (Block) и вставку блока (BlockReference)
                            ' 1. Объявим переменную типа Database и присвоим ей значение
                            ' используя свойство Document.Database объекта ed класса Editor
                            ' Заметка: Добавим пространство имён Autodesk.AutoCAD.DatabaseServices Namespace для Database
                            ' и Transaction используя ключевое слово Imports (выше объявления класса)
                            Dim dwg As Database = ed.Document.Database

                            ' 2. Объявим переменную типа Transaction и присвоим ей значение возвращаемое 
                            ' методом TransactionManager.StartTransaction.
                            Dim trans As Transaction = dwg.TransactionManager.StartTransaction()

                            ' 3. Добавим блок Try/Catch/Finally. Перенесём Catch перед шагом 9.
                            ' Поместим Finally перед шагом 10 и End Try после шага 10.  
                            ' 
                            Try

                                ' 4. Объявим переменную класса Circle и создадим её при помощи оператора New. 
                                ' Используем свойство Value переменной PromptPointResult созданной в
                                ' Lab2 в качестве первого параметра. В качестве второго параметра (нормаль) используем
                                ' Vector3d.ZAxis. Используем свойство Value объекта типа PromptDoubleResult
                                ' в качестве радиуса.
                                ' Заметка: Добавим пространство имён Autodesk.AutoCAD.Geometry для Vector3d
                                ' Воспользуемся ключевым словом Imports (перед объявлением класса)
                                Dim circle As New Circle(getPointResult.Value, Vector3d.ZAxis, getRadiusResult.Value)

                                ' 5. Объявим переменную типа BlockTableRecord. Присвоим ей значение используя 
                                ' метод GetObject переменной типа Transaction, созданной на шаге 2. 
                                ' Используем свойство CurrentSpaceId переменной класса Database, созданной на шаге 1
                                ' в качестве первого параметра. (ObjectId) В качестве второго параметра
                                ' используем OpenMode.ForWrite. Мы добавим окружность или к пространству
                                ' модели (ModelSpace) или к пространству листа (PaperSpace) - это определяет CurrentSpaceId)
                                Dim btr As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)

                                ' 6. Добавим  окружность (Circle) к BlockTableRecord определенной на шаге 5.
                                ' Воспользуемся методом AppendEntity и передадим ему окружность, созданную на шаге 4.
                                btr.AppendEntity(circle)

                                ' 7. Сообщим транзакции о новой окружности, так что она сама <автозакроет> окружность
                                ' Используем для этого метод AddNewlyCreatedDBObject method.
                                ' Первый её аргумент - circle. Второй - True.
                                trans.AddNewlyCreatedDBObject(circle, True)

                                ' 8. Завершим транзакцию при помощи вызова метода Commit.
                                trans.Commit()

                            Catch ex As Exception
                                ' 9. Объявим переменную типа Exception для Catch.
                                ' (добавим "ex as Exception" после ключевого слова Catch)
                                ' Воспользуемся методом WriteMessage переменной типа Editor (ed) из Lab2.
                                ' Используем "problem due to " + ex.Message в качестве параметра.
                                ' Если появятся ошибки - их детали будут напечатаны в командной строке AutoCAD.

                                ed.WriteMessage("problem due to " + ex.Message)
                            Finally
                                ' 10. Освободим транзакцию при помощи вызова метода Dispose объекта
                                ' типа Transaction созданной на шаге 2. Всё это будет вызвано в случае есть ли ошибки или их нет.
                                trans.Dispose()
                            End Try

                        End If
                    End If

                Case "Block"

                    ' вводим имя блока

                    Dim blockNameOptions As PromptStringOptions = New PromptStringOptions("Enter name of the Block to create : ")
                    ' запрещаем пробелы в имени блока
                    blockNameOptions.AllowSpaces = False
                    ' получаем имя

                    Dim blockNameResult As PromptResult = ed.GetString(blockNameOptions)
                    ' Если OK
                    If (blockNameResult.Status = PromptStatus.OK) Then

                        ' 11. Объявим переменную типа Database и присвоим ей значение равное
                        ' свойству Document.Database переменной ed класса Editor
                        Dim dwg As Database = ed.Document.Database


                        ' 12. Объявим переменную типа Transaction и присвоим ей значение
                        ' возвращаемое методом TransactionManager.StartTransaction.
                        Dim trans As Transaction = dwg.TransactionManager.StartTransaction

                        ' 13. Добавим Try/Catch/Finally блок. Перенесём оператор Catch перед шагом 36.
                        ' Поместим Finally перед шагом 37 и End Try после шага 37.
                        Try

                            ' Создаём новое определение блока
                            ' 14. Объявляем переменную типа BlockTableRecord. Создаём её при помощи оператора New.
                            Dim newBlockDef As BlockTableRecord = New BlockTableRecord

                            ' 15. Установим свойство Name для определения блока. Используем
                            ' свойство StringResult переменной типа PromptResult.
                            ' (создана в Lab2)
                            newBlockDef.Name = blockNameResult.StringResult

                            ' 16. Объявим переменную типа BlockTable. Присвоим ей значение, возвращаемое
                            ' методом GetObject объекта типа Transaction. В качестве первого параметра
                            ' используем свойство BlockTableId переменной типа Database, созданной в шаге 11
                            ' В качестве второго параметра используем OpenMode.ForRead.
                            ' Мы открываем блок <для чтения> если блок уже с таким именем есть.
                            Dim blockTable As BlockTable = trans.GetObject(dwg.BlockTableId, OpenMode.ForRead)

                            ' 17. Добавим оператор If then. Проверим есть ли уже такой блок в таблице
                            ' блоков при помощи метода Has для переменной определенной на шаге 16
                            ' В качестве строкового параметра используем свойство StringResult переменной
                            ' типа PromptResult, созданной раньше в Lab2.
                            ' Проверим равно ли оно False. Перенесём End if после шага 35
                            If (blockTable.Has(blockNameResult.StringResult) = False) Then

                                ' 18. Блок с таким именем не существует, так что добавим его.
                                ' Для начала сделаем BlockTable открытым для записи. Это можно сделать
                                ' вызовом метода UpgradeOpen() для BlockTable. (созданной на шаге 16)
                                blockTable.UpgradeOpen()

                                ' 19. Добавим BlockTableRecord созданную на шаге 14. Воспользуемся методом Add
                                ' объекта типа BlockTable и передадим ему в качестве параметра BlockTableRecord.
                                blockTable.Add(newBlockDef)

                                ' 20. Укажем транзакции новый блок, чтобы она его <автозакрыла>
                                ' Используем метод AddNewlyCreatedDBObject. Первый аргумент
                                ' BlockTableRecord. Для второго используем True.
                                trans.AddNewlyCreatedDBObject(newBlockDef, True)

                                ' 21. На следующих двух шагах добавим окружности к BlockTableRecord
                                ' Объявим переменную класса Circle и создадим её оператором New
                                ' Первуй аргумент: New Point3d(0, 0, 0). Второй - Vector3d.ZAxis
                                ' Используем 10 в качестве радиуса.
                                ' Заметка: Добавим пространство имён Autodesk.AutoCAD.Geometry для Point3d
                                ' Используем ключевое слово Imports (перед определением класса)
                                Dim circle1 As New Circle(New Point3d(0, 0, 0), Vector3d.ZAxis, 10)

                                ' 22. Добавим окружность к BlockTableRecord.
                                ' Воспользуемся методом AppendEntity и передадим окружность
                                ' созданную на предыдущем шаге
                                newBlockDef.AppendEntity(circle1)

                                ' 23. Добавим еще одну окружность к BlockTableRecord
                                ' Объявим переменную как Circle и создадим её оператором New
                                ' Первый аргумент: New Point3d(20,10,0)
                                ' Второй аргумент Vector3d.ZAxis
                                ' Радиус 10.
                                Dim circle2 As New Circle(New Point3d(20, 10, 0), Vector3d.ZAxis, 10)

                                ' 24. Добавим вторую окружность к BlockTableRecord используя
                                ' метод AppendEntity, передав ей окружность из шага 23
                                newBlockDef.AppendEntity(circle2)

                                ' 25. Укажем транзакции о новых объектах, чтобы она могла их <автозакрыть>
                                ' Вызовём метод the AddNewlyCreatedDBObject, передав окржность из шага 21.
                                ' Повторим это с окружностью из шага 23
                                ' Второй аргумент в обоих вызовах True.
                                trans.AddNewlyCreatedDBObject(circle1, True)
                                trans.AddNewlyCreatedDBObject(circle2, True)


                                ' 26. Мы создали определение блока (BlockTableRecord).
                                ' Сейчас мы воспользуемся этим блоком для вставки
                                ' блока (BlockReference) текущее пространство.
                                ' Сначала объявим переменную типа PromptPointOptions создадим её оператором New
                                ' В качестве параметра message используем "Pick insertion point of Block : "
                                Dim blockRefPointOptions As PromptPointOptions = New PromptPointOptions("Pick insertion point of BlockRef : ")

                                ' 27. Объявим переменную типа PromptPointResult. Используем метод GetPoint
                                ' объекта Editor (из Lab2). Передадим этому методу переменную типа PromptPointOptions из шага 26
                                Dim blockRefPointResult As PromptPointResult = ed.GetPoint(blockRefPointOptions)

                                ' 28. Создадим блок If then и проверим Status переменной PromptPointResult.
                                ' Проверим, не равен ли он PromptStatus.OK.
                                ' Поместим блок End If после шага 30
                                If (blockRefPointResult.Status <> PromptStatus.OK) Then

                                    ' 29. Если мы попали внутрь блока, то вызов метода GetPoint не удался.
                                    ' Вызовем метод Dispose() транзакции, созданной на шаге 11
                                    trans.Dispose()
                                    ' 30. Return 
                                    Return
                                End If

                                ' 31. Объявим переменную типа BlockReference. Создадим её при помощи оператора New
                                ' Используем метод Value переменно типа PromptPointResult в качестве аргумента Position (Point3d)
                                ' Используем свойство ObjectId объекта BlockTableRecord из шаша 14 в
                                ' качестве второго параметра.
                                Dim blockRef As BlockReference = New BlockReference(blockRefPointResult.Value, newBlockDef.ObjectId)

                                ' 32. Получим текущее пространство(или Модели или Листа).
                                ' Объявим переменную типа BlockTableRecord и создадим её при помощи метода
                                ' GetObject транзакции, созданной на шаге 12. Используем свойство
                                ' CurrentSpaceId объекта Database созданного на шаге 11.
                                ' Открываем его сразу для записи (OpenMode.ForWrite)
                                Dim curSpace As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)

                                ' 33. Используем метод AppendEntity объекта BlockTableRecord,
                                ' созданного на шаге 32 и передадим ему BlockReference с шага 31.
                                curSpace.AppendEntity(blockRef)

                                ' 34. Сообщим транзакции о новой вставке блока, чтобы транзакция его
                                ' <автозакрыла> - метод AddNewlyCreatedDBObject объекта Transaction, созданного на шаге 12
                                ' Передаём в него BlockReference, второй параметр - True
                                trans.AddNewlyCreatedDBObject(blockRef, True)

                                ' 35. Если мы сюда попали - всё в порядке. Завершаем транзакцию, вызывая её метод Commit
                                trans.Commit()
                            End If
                        Catch ex As Exception
                            ' 36. Объявим перенную типа Exception в блоке Catch.
                            ' (добавим "ex as Exception" после ключевого слова Catch)
                            ' Воспользуемся методом WriteMessage объекта класса Editor (ed) из Lab2.
                            ' Папаметром будет "a problem occured because " + ex.Message.
                            ' Если появятся ошибки - их детали будут напечатаны в командной строке AutoCAD.

                            ed.WriteMessage("a problem occured because " + ex.Message)
                        Finally
                            ' 37. Освободим транзакцию, вызвав метод Dispose
                            ' объекта класса Transaction созданного на шаге 12.
                            ' Он будет вызван вне зависимости от того были ли ошибки или нет.
                            trans.Dispose()

                        End Try

                    End If

            End Select
        End If

    End Sub
End Class