﻿' (C) Copyright 2002-2005 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted, 
' provided that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting 
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports System.Windows

Imports Autodesk.AutoCAD.Windows

Public Class adskClass

    ' Определяем команду 'helloworld'
    <CommandMethod("helloworld")> _
    Public Sub helloworld()

        ' Получаем объект редактора
        Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
        ' Пишем в командной строке
        ed.WriteMessage(vbCr + "Здравствуй, Мир!")

    End Sub

    <CommandMethod("addAnEnt")> _
    Public Sub AddAnEnt()

        ' Получаем объект редактора т.к. собираемся кое-что вводить
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ' уточним какой тип примитива будем создавать
        Dim getWhichEntityOptions As PromptKeywordOptions = New PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block")
        ' теперь запросим его
        Dim getWhichEntityResult As PromptResult = ed.GetKeywords(getWhichEntityOptions)
        ' Если OK
        If (getWhichEntityResult.Status = PromptStatus.OK) Then

            ' Проверим какой тип примитива требуется создать
            Select Case getWhichEntityResult.StringResult

                Case "Circle"

                    ' Укажем центр окружности
                    Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick Center Point : ")
                    Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)
                    ' Если OK
                    If (getPointResult.Status = PromptStatus.OK) Then

                        ' получаем радиус
                        Dim getRadiusOptions As PromptDistanceOptions = New PromptDistanceOptions("Pick Radius : ")
                        ' в качестве базовой точки - точку которую мы указали для центра
                        getRadiusOptions.BasePoint = getPointResult.Value
                        ' сообщим механизму ввода использовать базовую точку
                        getRadiusOptions.UseBasePoint = True
                        ' теперь получим радиус
                        Dim getRadiusResult As PromptDoubleResult = ed.GetDistance(getRadiusOptions)
                        ' Если всё OK
                        If (getRadiusResult.Status = PromptStatus.OK) Then

                            ' Нужно добавить окружность к текущему пространству
                            ' Получаем активную базу
                            Dim dwg As Database = ed.Document.Database
                            ' стартуем транзакцию
                            Dim trans As Transaction = dwg.TransactionManager.StartTransaction()
                            Try

                                ' создаём новую окружность
                                Dim circle As New Circle(getPointResult.Value, Vector3d.ZAxis, getRadiusResult.Value)
                                ' открываем активное пространство (запись таблицы блоков) для записи
                                Dim btr As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                                ' добавляем окружность к текущему пространству (скорее всего пространство модели)
                                btr.AppendEntity(circle)
                                ' сообщим транзакции о необходимости автозакрыть объект окружности
                                trans.AddNewlyCreatedDBObject(circle, True)
                                ' завершаеи (commit) транзакцию
                                trans.Commit()

                            Catch ex As Exception
                                ' Есть исключение - сообщим о нём
                                ed.WriteMessage("problem due to " + ex.Message)
                            Finally
                                ' Дело сделано, в любом случае освободим транзакцию
                                trans.Dispose()
                            End Try

                        End If
                    End If

                Case "Block"

                    ' вводим имя блока
                    Dim blockNameOptions As PromptStringOptions = New PromptStringOptions("Enter name of the Block to create : ")
                    ' запрещаем пробелы в имени блока
                    blockNameOptions.AllowSpaces = False
                    ' получаем имя
                    Dim blockNameResult As PromptResult = ed.GetString(blockNameOptions)
                    ' Если ok
                    If (blockNameResult.Status = PromptStatus.OK) Then

                        ' Создадим определение блока
                        ' получаем активный чертеж
                        Dim dwg As Database = ed.Document.Database
                        ' начнем транзакцию
                        Dim trans As Transaction = dwg.TransactionManager.StartTransaction
                        Try

                            ' создаем новое определение блока
                            Dim newBlockDef As BlockTableRecord = New BlockTableRecord
                            ' задаём имя блоку
                            newBlockDef.Name = blockNameResult.StringResult

                            ' добавим определение блока в таблицу блоков
                            ' откроем таблицу блоков для чтения и проверим есть ли уже такое имя блока в ней
                            Dim blockTable As BlockTable = trans.GetObject(dwg.BlockTableId, OpenMode.ForRead)
                            ' проверяем есть ли уже такое имя
                            If (blockTable.Has(blockNameResult.StringResult) = False) Then
                                ' Если нет, то можем добавить его
                                ' только сначала нужно переключится в режим для записи
                                blockTable.UpgradeOpen()
                                blockTable.Add(newBlockDef)
                                ' сообщаем менеджеру транзакций о новом объекте для автозакрытия
                                trans.AddNewlyCreatedDBObject(newBlockDef, True)
                            End If

                            ' теперь добавим несколько объектов к определению блока
                            Dim circle1 As New Circle(New Point3d(0, 0, 0), Vector3d.ZAxis, 10)
                            Dim circle2 As New Circle(New Point3d(20, 10, 0), Vector3d.ZAxis, 10)
                            Dim circle3 As New Circle(New Point3d(40, 20, 0), Vector3d.ZAxis, 10)
                            Dim circle4 As New Circle(New Point3d(60, 30, 0), Vector3d.ZAxis, 10)
                            Dim circle5 As New Circle(New Point3d(80, 40, 0), Vector3d.ZAxis, 10)
                            newBlockDef.AppendEntity(circle1)
                            newBlockDef.AppendEntity(circle2)
                            newBlockDef.AppendEntity(circle3)
                            newBlockDef.AppendEntity(circle4)
                            newBlockDef.AppendEntity(circle5)
                            ' сообщим менеджеру транзакций о новых объектах
                            trans.AddNewlyCreatedDBObject(circle1, True)
                            trans.AddNewlyCreatedDBObject(circle2, True)
                            trans.AddNewlyCreatedDBObject(circle3, True)
                            trans.AddNewlyCreatedDBObject(circle4, True)
                            trans.AddNewlyCreatedDBObject(circle5, True)

                            ' теперь вставим блок в текущее пространство - запросим точку вставки
                            Dim blockRefPointOptions As PromptPointOptions = New PromptPointOptions("Pick insertion point of Block : ")
                            Dim blockRefPointResult As PromptPointResult = ed.GetPoint(blockRefPointOptions)
                            ' если не всё в порядке
                            If (blockRefPointResult.Status <> PromptStatus.OK) Then
                                ' освобождаем транзакцию, чтобы отменить изменения и выходим
                                trans.Dispose()
                                Return
                            End If
                            ' теперь у нас есть определение блока и точка для вставки - создадим вставку блока
                            Dim blockRef As BlockReference = New BlockReference(blockRefPointResult.Value, newBlockDef.ObjectId)
                            ' открываем текущее пространство для записи и добавим вставку блока в него
                            Dim curSpace As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                            ' добавляем вставку блока
                            curSpace.AppendEntity(blockRef)
                            ' укажем транзакции о новом объекте для автозакрытия
                            trans.AddNewlyCreatedDBObject(blockRef, True)

                            ' Если всё в норме - завершим транзакцию
                            trans.Commit()

                        Catch ex As Exception
                            ' Есть проблема - сообщим о ней
                            ed.WriteMessage("a problem occured because " + ex.Message)
                        Finally
                            ' чтобы не произошло, а транзакцию нужно освободить
                            trans.Dispose()

                        End Try

                    End If

            End Select
        End If

    End Sub


    ' 1. Добавим ссылку на PresentationCore. (на вкладке .NET 
    ' диалога Add Reference. Это необходимо для Палитры (PaletteSet)
    ' которую мы объявим на шаге 3.

    ' 2. Используем директиву Imports для пространства имён Autodesk.AutoCAD.Windows

    ' 3. Объявим глобальную переменную класса PaletteSet. (Она должна
    ' быть создана толко один раз). Добавим это объявление после 
    ' оператора End Sub процедура AddAnEnt из Lab 3. 
    Public myPaletteSet As PaletteSet

    ' 4. Объявим переменную типа UserControl1. Эта переменная - 
    ' контрол, создана в самом начале Lab 4
    ' Этот контрол будет размещен на Палитре (PaletteSet), созданной на шаге 3.
    Public myPalette As UserControl1

    ' 5. Добавим новую команду с именем "palette". Воспользуемся
    ' атрибутом CommandMethod и создадим процедуру, которая 
    ' будет выполнятся при вызове команды в AutoCAD
    ' Заметка: Поместим End Sub после шага 10.
    <CommandMethod("palette")> _
    Public Sub palette()

        ' 6. Добавим оператор "if then" и проверим равна ли
        ' PaletteSet объявленная на шаге 3 Nothing (т.е. ничему)
        ' Это возможно при первом запуске команды
        ' Заметка: Поместим "End If" после шага 9
        If (myPaletteSet = Nothing) Then

            ' 7. Значение PaletteSet не присвоено - создадим новую
            ' с уникальным идентификатором GUID. Используем оператор New. 
            ' Параметр имя передаем строкой "My Palette". В качестве
            ' параметра ToolID генерируем новый GUID.
            ' В меню Tools выбираем "Create Guid". В диалоге Create GUID
            ' выбираем "Registry Format" Выбираем New GUID и копируем.
            ' Вставляем этот GUID для использования в качестве параметра
            ' для New Guid.
            ' Заменяем фигурные скобки на двойные кавычки 
            ' (Параметр для New Guid должен быть строковый) 
            myPaletteSet = New PaletteSet("My Palette", New Guid("D61D0875-A507-4b73-8B5F-9266BEACD596"))

            ' 8. Создаём переменную класса UserControl1 объявленую на шаге 4
            ' Используем оператор New (New UserControl1) 
            ' На этом контроле будет размещен контрол TreeView.
            myPalette = New UserControl1

            ' 9. Добавим UserControl к PaletteSet. Используем метод Add 
            ' объекта PaletteSet, созданного на шаге 7. Передадим "Palette1" 
            ' в качестве первого параметра и контрол, созданный на шаге 8,
            ' в качестве второго параметра
            myPaletteSet.Add("Palette1", myPalette)

        End If

        ' 10. Покажем Палитру установив её свойство Visible в True 
        ' При следующем выполнении команды только этот 
        ' код и будет выполнен
        myPaletteSet.Visible = True

    End Sub


    ' 11. Добавим команду с именем "addDBEvents". Используем атрибут 
    ' CommandMethod и добавляем процедуру, которая будет выполняться 
    ' при запуске команды вn AutoCAD
    ' Заметка: Поместимe End Sub после шага 20
    <CommandMethod("addDBEvents")> _
    Public Sub addDBEvents()

        ' 12. Используем оператор If Then для проверки равна ли palette
        ' созданная на шаге 4  Nothing.
        ' Заметка: Поместите End If после шага 15
        If myPalette Is Nothing Then
            ' 13. Объявим и создадим объект редактора (Editor). 
            ' Воспользуемся свойством Editor объекта 
            ' Application.DocumentManager.MdiActiveDocument
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            ' 14. Используем метод WriteMessage переменной типа Editor  из
            ' шага 13. В качестве параметра message
            ' vbCr + "Please call the 'palette' command first"
            ed.WriteMessage(vbCr + "Please call the 'palette' command first")

            ' 15. Выходим из процедуры

            Exit Sub

        End If

        ' 16. Объявим переменную класса Database и присвоим ей значение, 
        ' равное свойству Database объекта 
        ' Application.DocumentManager.MdiActiveDocument
        Dim curDwg As Database = Application.DocumentManager.MdiActiveDocument.Database

        ' 17. Используем оператор AddHandler для регистрации обработчика
        ' события ObjectAppended
        ' Используем событие ObjectAppended переменной типа Database из шага 16 
        ' в качестве первого параметра (Event). В качестве второго
        ' параметра (Delegate) используем оператор New и создаём
        ' ObjectEventHandler. 
        ' Используем оператор AddressOf и имя процедуры (callback_ObjectAppended)
        ' которую создадим на шаге 21. 
        AddHandler curDwg.ObjectAppended, New ObjectEventHandler(AddressOf callback_ObjectAppended)

        ' 18. Используем оператор AddHandler для события ObjectErased
        ' объекта Database, созданного на шаге 16 в качестве первого
        ' параметра (Event). Для второго параметра (Delegate) 
        ' воспользуемся оператором New и создадим ObjectErasedEventHandler.
        ' Используем оператор AddressOf для имени процедцры (callback_ObjectErased)
        ' которую мы создадим на шаге 24. 
        AddHandler curDwg.ObjectErased, New ObjectErasedEventHandler(AddressOf callback_ObjectErased)

        ' 19. Используем оператор AddHandler для события ObjectReappended
        ' объекта Database, созданного на шаге 16 в качестве первого
        ' параметра (Event). Для второго параметра (Delegate) 
        ' воспользуемся оператором New и создадим ObjectEventHandler.
        ' Используем оператор AddressOf для имени процедуры (callback_ObjectReappended)
        ' которую мы создадим на шаге 32. 
        AddHandler curDwg.ObjectReappended, New ObjectEventHandler(AddressOf callback_ObjectReappended)

        ' 20. Используем оператор AddHandler для события ObjectUnappended
        ' объекта Database, созданного на шаге 16 в качестве первого
        ' параметра (Event). Для второго параметра (Delegate) 
        ' воспользуемся оператором New и создадим ObjectEventHandler.
        ' Используем оператор AddressOf для имени процедцры (callback_ObjectUnappended)
        ' которую мы создадим на шаге 35. 
        AddHandler curDwg.ObjectUnappended, New ObjectEventHandler(AddressOf callback_ObjectUnappended)

    End Sub

    ' 21. Создадим Private Sub с именем callback_ObjectAppended. Эта 
    ' процедура будет вызываться, когда объект будет добавлятся в
    ' базу данных (Имя процедуры должно совпадать с именем параметра
    ' Delegate на шаге 17). Первый параметр - объект. 
    ' (Используем ByVal и sender в качестве имени объекта).
    ' Второй аргумент - объект класса ObjectEventArgs. 
    ' (Используем ByVal и в качестве имени - "e")
    ' Заметка: Поместим End Sub после шага 23
    Private Sub callback_ObjectAppended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' 22. Объявим переменную класса TreeNode (System.Windows.Forms.TreeNode). 
        ' Заметка: Вы можете сэкономить несколько нажатий 
        ' на клавиши с помощью Imports - импорта пространства имен
        ' Создадим её с помощью метода Add для свойства Nodes объекта 
        ' типа TreeView в UserForm() созданной на шаге 4.
        ' Используем ObjectEventArgs переданный в метод в качестве
        ' строкового параметра и используем тип DBObject. 
        ' (e.DBObject.GetType().ToString())
        Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())

        ' 23. Присваиваем свойство Tag для node созданного на шаге 22
        ' равным ObjectId добавленного объекта. Это позволит нам
        ' сохранить его ObjectId для идентификации его в других событиях 
        ' Используем e.DBObject.ObjectId.ToString()
        newNode.Tag = e.DBObject.ObjectId.ToString()

    End Sub

    ' 24. Создаём Private Sub с именем callback_ObjectErased. Эта
    ' процедура будет вызываться при удалении объектов из базы
    ' (Имя должно быть тем, что мы указали в качестве параметра на шаге 18)
    ' Первый параметр - объект
    ' (Используем ByVal и sender в качестве имени объекта).
    ' Второй аргумент - объект класса ObjectErasedEventArgs. 
    ' (Используем ByVal и в качестве имени - "e")
    ' Заметка: Поместим End Sub перед шагом 32
    Private Sub callback_ObjectErased(ByVal sender As Object, ByVal e As ObjectErasedEventArgs)

        ' 25. Используем оператор If Then Else и проверим свойство
        ' Erased объекта ObjectErasedEventArgs, в функцию (e.Erased) 
        ' Заметка: Поместим оператор "Else" перед шагом 30 и "End If"
        ' после шага 31
        If e.Erased Then

            ' 26. Здесь мы поищем объект в контроле TreeView, т.к. его можно удалить.
            ' Запишем оператор For Each. Используем  node в качестве имени 
            ' элемента и тип Forms.Treenode. Групповой параметр - Nodes в TreeView.
            ' (myPalette.TreeView1.Nodes)
            ' Заметка: Поместим оператор Next после шага 29. (перед оператором Else)
            For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
                ' 27. Используем "If Then" оператор. Сравниваем свойство Tag переменной node и ObjectId
                ' удаленного объекта. Используем свойство DBObject параметра 
                ' ObjectErasedEventArgs переданного в событие (e.DBObject.ObjectId.ToString)
                ' Заметка: поместим "End If" над оператором "Next" добавленном в шаге 23
                If (node.Tag = e.DBObject.ObjectId.ToString) Then
                    ' 28. Удаляем node при помощи вызова метода Remove. (Примитив удалён из чертежа).
                    node.Remove()
                    ' 29. Выход из цикла. (Exit For)
                    Exit For
                End If
            Next

        Else
            ' 30. Если это происходит, значит объект восстановлен
            ' (unerased). (e.Erased равно False)
            ' Объявим System.Windows.Forms.TreeNode с именем newNode 
            ' Создадим его при помощи метода Add коллекции Nodes
            ' объекта TreeView созданного на предыдущих шагах.
            ' Используем тип объекта в качестве параметра.
            ' e.DBObject.GetType().ToString()
            Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())

            ' 31. Устанавливаем свойство Tag для newNode равным ObjectId 
            ' восстановленного объекта. Это позволит нам записать его
            ' ObjectId для опознания его в других событиях
            ' Используем e.DBObject.ObjectId.ToString()
            newNode.Tag = e.DBObject.ObjectId.ToString()
        End If
    End Sub

    ' 32. Создаём Private Sub с именем callback_ObjectReappended. 
    ' Эта процедура будет вызываться, когда объект повторно 
    ' добавляться (ReAppended) в базу Database.
    ' (Имя должно быть таким, как задано в параметре Delegate 
    ' на шаге 19). Первый её параметр object. 
    ' (Используем ByVal и имя sender). Второй параметр типа
    ' ObjectEventArgs. (Используем ByVal и e в качестве имени)
    ' Заметка: Поместим End Sub после шага 34
    Private Sub callback_ObjectReappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

        ' 33. Добавим имя класса объекта к TreeView
        ' Объявим переменную типа TreeNode (System.Windows.Forms.TreeNode).
        ' Создадим её используя метод Add для свойства Nodes контрола TreeView в UserForm1
        ' созданной на шаге 4. Используем переданную в этот метод
        ' ObjectEventArgs в качестве строкового параметра
        ' и используем тип DBObject. (e.DBObject.GetType().ToString())
        Dim newNode As Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())

        ' 34. Запишем её id для дальнейшего опознания
        ' Делаем свойство Tag для node, созданного на шаге 33, 
        ' равным ObjectId восстановленного объекта
        ' Это позволит нам в дальнейшем записать его ObjectId 
        ' для опознания в других событиях. 
        ' Используем e.DBObject.ObjectId.ToString()
        newNode.Tag = e.DBObject.ObjectId.ToString()

    End Sub

    ' 35. Создаём Private Sub с именем callback_ObjectUnappended. 
    ' Эта процедура вызывается когда объект UnAppended из базы данных. 
    ' (Имя процедуры должно совпадать с именем параметра Delegate 
    ' из шага 20). Первый параметр - объект
    ' (Используем ByVal и имя sender в качестве имени объекта).
    ' Второй параметр типа ObjectEventArgs. (Используем ByVal и имя
    ' e в качестве имени ObjectEventArgs)
    ' Заметка: Поместим End Sub после шага 39
    Private Sub callback_ObjectUnappended(ByVal sender As Object, ByVal e As ObjectEventArgs)
        ' 36. Здесь мы будем искать объект в TreeView т.к. он может быть удалён.
        ' Создаём оператор For Each. Используем имя node в качестве 
        ' имени элемента и тип его Forms.Treenode. Параметр группы - 
        ' свойство Nodes в TreeView. (myPalette.TreeView1.Nodes)
        ' Заметка: Поместим оператор Next после шага 39 
        For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
            ' 37. Используем оператор "If Then" и смотрим тот ли node,
            ' который мы исчем. Сравниваем node.Tag и ObjectId. 
            ' (используем e.DBObject.ObjectId.ToString)
            ' Заметка: Поместим "End If" после шага 39 
            If (node.Tag.ToString = e.DBObject.ObjectId.ToString) Then
                ' 38. Если мы нашли, что это node объекта, который unappended.
                ' вызовем метод его метод Remove.
                node.Remove()
                ' 39. Выйдем из цикла.
                Exit For
            End If
        Next

    End Sub

End Class
